###########################################################################################################################
## Chernozhukov, Fernandez-Val, Hahn, and Newey: "Average and Quantile Effects in Nonseparable Panel Models"
## R program to generate Figures 4B & 4C
###########################################################################################################################

# DATA: NLSY 1986-1993 (2.065 MEN)

# Variables:
#    1 - y: log-wage in $1980
#    2 - x: union status binary indicator
#    3 - tx: type of individual with respect to history of x using binary coding, 1 + sum_t 2^{x_t}
#    4 - tx1: type of individual with respect to union status in first period, = 1 if x_1 = 1
#    5 - xf1: individual indicator for first period unionized
#    6 - xf0: individual indicator for first period non-unionized
#    7 - xl1: individual indicator for unionized in the last period
#    8 - xl0: individual indicator for non-unionized in the last period


# This version: 08/28/2012

#####################################################
# PART I: PRELIMINARY FUNCTIONS
#####################################################

rm(list = ls());

############## obtains the distribution of (uniform) critical value by bootstrap ##############################################

subsamplek <- function(data=data, n=n, B=B, taus, T)
    {

    bootstrap 		<- NULL;
    uqet.boots		<- NULL;
    lqet.boots		<- NULL;
    qei.boots		<- NULL;
    uqetm.boots		<- NULL;
    lqetm.boots		<- NULL;

    uqetd.boots		<- NULL;
    lqetd.boots		<- NULL;
    qeid.boots		<- NULL;
    uqetmd.boots	<- NULL;
    lqetmd.boots	<- NULL;


    for (s in 1:B)
        {
	sdata		<- data[(kronecker(sample(n,n,replace=T), matrix(1,nrow=T))-1)*T + matrix(c(1:T), nrow = n*T), ];

  # Location and scale time effects;

	meant	<- NULL;
  st   <- NULL;

	for (period in 1:T) {;
		meant[period]	<- mean(sdata$y[(sdata$tx1*sdata$x + (1-sdata$tx1)*(1-sdata$x)) == 1 & sdata$time == period]);
    st[period]    <- sum(sdata$y[sdata$tx1*sdata$x == 1 & sdata$time == period] - meant[period]);                  
		};

  scale <- st/st[1];
  loc <-  meant - scale*meant[1];
  
	yte	<- (sdata$y - kronecker(matrix(1, nrow = n), loc) ) / kronecker(matrix(1, nrow = n), scale) ;


	# Creating indicators and smoothed indicators to estimate cdf;

	ys	<- quantile(yte, c(0:1000)/1000);
	y_ind.s	<- pnorm(matrix(kronecker(matrix(1, nrow = length(yte), ncol = 1), matrix(ys, ncol = 1)) - kronecker(matrix(yte, ncol = 1), matrix(1, nrow = length(ys), ncol = 1)), nrow = length(yte), ncol = length(ys), byrow = T)/ bw.nrd0(yte)); 

  # Creating matrices to estimate bounds of average effects on treated or first treated;
 
  pA_x1		<- NULL;
	A1_x1.s		<- NULL;
	A0_x1.s		<- NULL;
  pA_x1.d		<- NULL;
	A1_x1.s.d	<- NULL;
	A0_x1.s.d	<- NULL;

	for (i in 2:2^T) {;
		pA_x1		  <- rbind(pA_x1, mean(sdata$tx == i) / mean(sdata$tx != 1));
  	pA_x1.d		<- rbind(pA_x1.d, mean(sdata$tx == i & sdata$tx1 == 1) / mean(sdata$tx != 1 & sdata$tx1 == 1));
		A1_x1.s		<- rbind(A1_x1.s, apply(kronecker(matrix(sdata$x[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(sdata$x[sdata$tx == i]), 1));
		A0_x1.s		<- rbind(A0_x1.s, apply(kronecker(matrix(1 - sdata$x[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(1 - sdata$x[sdata$tx == i]), 1));
		A1_x1.s.d	<- rbind(A1_x1.s.d, apply(kronecker(matrix(sdata$xf1[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(sdata$xf1[sdata$tx == i]), 1));
		A0_x1.s.d	<- rbind(A0_x1.s.d, apply(kronecker(matrix(sdata$xf0[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(sdata$xf0[sdata$tx == i]), 1));
		};

	AA1.s	<- t(pA_x1) %*% A1_x1.s; 
	AA0.s	<- t(pA_x1) %*% A0_x1.s; 
	AA1.s.d	<- t(pA_x1.d) %*% A1_x1.s.d; 
	AA0.s.d	<- t(pA_x1.d) %*% A0_x1.s.d; 

	# Creating matrices to estimate bounds of average effects on treated or first treated under monotonicity;

	AA1.m.s	<- t(pA_x1) %*% A1_x1.s; 
	AA0.m.s	<- t(pA_x1) %*% rbind(A0_x1.s[-(2^T-1),],A1_x1.s[2^T-1,]); 

	pA_x1.d		<- NULL;
	A1_x1.s.d	<- NULL;
	A0_x1.s.d	<- NULL;

	for (i in 2:2^T) {;
		pA_x1.d		<- rbind(pA_x1.d, mean(sdata$tx == i & sdata$tx1 == 1) / mean(sdata$tx != 1 & sdata$tx1 == 1));
		A1_x1.s.d	<- rbind(A1_x1.s.d, apply(kronecker(matrix(sdata$xf1[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(sdata$xf1[sdata$tx == i]), 1));
		A0_x1.s.d	<- rbind(A0_x1.s.d, apply(kronecker(matrix(sdata$xf0[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(sdata$xf0[sdata$tx == i]), 1));
		};

	A0_x1.s.d	<- rbind(A0_x1.s.d[-(2^T-1),], apply(kronecker(matrix(sdata$xl1[sdata$tx == 2^T], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == 2^T], 2, sum) / max(sum(sdata$xl1[sdata$tx == 2^T]), 1) );
	AA1.m.s.d	<- t(pA_x1.d) %*% A1_x1.s.d; 
	AA0.m.s.d	<- t(pA_x1.d) %*% A0_x1.s.d; 


	# Creating matrices to estimate identified effects;


	pA_xi		<- NULL;
	A1_xi.s		<- NULL;
	A0_xi.s		<- NULL;
	A1_xi.s.d	<- NULL;
	A0_xi.s.d	<- NULL;

	for (i in 2:(2^T-1)) {;
		pA_xi		<- rbind(pA_xi, mean(sdata$tx == i) / mean(sdata$tx != 1 & sdata$tx != 2^T));
		A1_xi.s		<- rbind(A1_xi.s, apply(kronecker(matrix(sdata$x[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(sdata$x[sdata$tx == i]), 1));
		A0_xi.s		<- rbind(A0_xi.s, apply(kronecker(matrix(1 - sdata$x[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(1 - sdata$x[sdata$tx == i]), 1));
		A1_xi.s.d	<- rbind(A1_xi.s.d, apply(kronecker(matrix(sdata$xf1[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(sdata$xf1[sdata$tx == i]), 1));
		A0_xi.s.d	<- rbind(A0_xi.s.d, apply(kronecker(matrix(sdata$xf0[sdata$tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[sdata$tx == i], 2, sum) / max(sum(sdata$xf0[sdata$tx == i]), 1));
		};

	AAA1.s		<- t(pA_xi) %*% A1_xi.s; 
	AAA0.s		<- t(pA_xi) %*% A0_xi.s;
	AAA1.s.d	<- t(pA_xi) %*% A1_xi.s.d; 
	AAA0.s.d	<- t(pA_xi) %*% A0_xi.s.d;


	# Distribution and quantile effects;


	uqt1.s		<- NULL;
	lqt1.s		<- NULL;
	uqt0.s		<- NULL;
	lqt0.s		<- NULL;

	qi1.s		<- NULL;
	qi0.s		<- NULL;

	uqt1.s.d	<- NULL;
	lqt1.s.d	<- NULL;
	uqt0.s.d	<- NULL;
	lqt0.s.d	<- NULL;

	qi1.s.d		<- NULL;
	qi0.s.d		<- NULL;



	for (tau in taus) {;

		lqt1.s	<- c(lqt1.s, ys[sum(AA1.s <= max(0, tau))] );
		uqt1.s	<- c(uqt1.s, ys[sum(AA1.s <= max(0, tau))] );
		lqt0.s	<- c(lqt0.s, ys[max(sum(AA0.s <= max(0, tau - pA_x1[2^T-1,1])), 1)] );
		uqt0.s	<- c(uqt0.s, ys[max(sum(AA0.s <= max(0, tau)), 1)] );

		qi1.s	<- c(qi1.s, ys[sum(AAA1.s <= max(0, tau))] );
		qi0.s	<- c(qi0.s, ys[max(sum(AAA0.s <= max(0, tau)), 1)] );


		lqt1.s.d	<- c(lqt1.s.d, ys[sum(AA1.s.d <= max(0, tau))] );
		uqt1.s.d	<- c(uqt1.s.d, ys[sum(AA1.s.d <= max(0, tau))] );
		lqt0.s.d	<- c(lqt0.s.d, ys[max(sum(AA0.s.d <= max(0, tau - pA_x1.d[2^T-1,1])), 1)] );
		uqt0.s.d	<- c(uqt0.s.d, ys[max(sum(AA0.s.d <= max(0, tau)), 1)] );

		qi1.s.d		<- c(qi1.s.d, ys[sum(AAA1.s.d <= max(0, tau))] );
		qi0.s.d		<- c(qi0.s.d, ys[max(sum(AAA0.s.d <= max(0, tau)), 1)] );


		};

	
	lqt1a.s		<- replace(lqt1.s, lqt1.s == ys[sum(AA1.s <= 0)], NA); 
	uqt1a.s		<- replace(uqt1.s, uqt1.s == ys[length(ys)], NA); 
	lqt0a.s		<- replace(lqt0.s, lqt0.s == ys[sum(AA0.s <= 0)], NA); 
	uqt0a.s		<- replace(uqt0.s, uqt0.s == ys[length(ys)], NA); 

	qi1a.s		<- replace(qi1.s, qi1.s == ys[sum(AAA1.s <= 0)], NA); 
	qi1a.s		<- replace(qi1.s, qi1.s == ys[length(ys)], NA); 
	qi0a.s		<- replace(qi0.s, qi0.s == ys[sum(AAA0.s <= 0)], NA); 
	qi0a.s		<- replace(qi0.s, qi0.s == ys[length(ys)], NA); 

	lqt0a.s		<- replace(lqt0.s, taus <= pA_x1[2^T-1,1], NA); 
	uqt0a.s		<- replace(uqt0.s, taus >= 1 - pA_x1[2^T-1,1], NA); 

	lqt1a.s.d	<- replace(lqt1.s.d, lqt1.s.d == ys[sum(AA1.s.d <= 0)], NA); 
	uqt1a.s.d	<- replace(uqt1.s.d, uqt1.s.d == ys[length(ys)], NA); 
	lqt0a.s.d	<- replace(lqt0.s.d, lqt0.s.d == ys[sum(AA0.s.d <= 0)], NA); 
	uqt0a.s.d	<- replace(uqt0.s.d, uqt0.s.d == ys[length(ys)], NA); 

	qi1a.s.d	<- replace(qi1.s.d, qi1.s.d == ys[sum(AAA1.s.d <= 0)], NA); 
	qi1a.s.d	<- replace(qi1.s.d, qi1.s.d == ys[length(ys)], NA); 
	qi0a.s.d	<- replace(qi0.s.d, qi0.s.d == ys[sum(AAA0.s.d <= 0)], NA); 
	qi0a.s.d	<- replace(qi0.s.d, qi0.s.d == ys[length(ys)], NA); 

	lqt0a.s.d	<- replace(lqt0.s.d, taus <= pA_x1.d[2^T-1,1], NA); 
	uqt0a.s.d	<- replace(uqt0.s.d, taus >= 1 - pA_x1.d[2^T-1,1], NA); 


	# Bounds for quantile effect on treated;

	uqet.s		<- uqt1a.s - lqt0a.s;
	lqet.s		<- lqt1a.s - uqt0a.s;

	uqet.s.d	<- uqt1a.s.d - lqt0a.s.d;
	lqet.s.d	<- lqt1a.s.d - uqt0a.s.d;

	# Identified effect;

	qei.s	<- qi1a.s - qi0a.s;
	qei.s.d	<- qi1a.s.d - qi0a.s.d;


	# Constructing bounds for quantile effects imposing monotonicity;


	uqt1.m.s	<- NULL;
	lqt1.m.s	<- NULL;
	uqt0.m.s	<- NULL;
	lqt0.m.s	<- NULL;

	uqt1.m.s.d	<- NULL;
	lqt1.m.s.d	<- NULL;
	uqt0.m.s.d	<- NULL;
	lqt0.m.s.d	<- NULL;



	for (tau in taus) {;

		lqt1.m.s	<- c(lqt1.m.s, ys[sum(AA1.m.s <= max(0, tau))] );
		uqt1.m.s	<- c(uqt1.m.s, ys[sum(AA1.m.s <= max(0, tau))] );
		lqt0.m.s	<- c(lqt0.m.s, ys[max(sum(AA0.m.s <= max(0, tau)), 1)] );
		uqt0.m.s	<- c(uqt0.m.s, ys[max(sum(AA0.m.s <= max(0, tau)), 1)] );

		lqt1.m.s.d	<- c(lqt1.m.s.d, ys[sum(AA1.m.s.d <= max(0, tau))] );
		uqt1.m.s.d	<- c(uqt1.m.s.d, ys[sum(AA1.m.s.d <= max(0, tau))] );
		lqt0.m.s.d	<- c(lqt0.m.s.d, ys[max(sum(AA0.m.s.d <= max(0, tau)), 1)] );
		uqt0.m.s.d	<- c(uqt0.m.s.d, ys[max(sum(AA0.m.s.d <= max(0, tau)), 1)] );

		};


	lqt1a.m.s	<- replace(lqt1.m.s, lqt1.m.s == ys[sum(AA1.m.s <= 0)], NA); 
	uqt1a.m.s	<- replace(uqt1.m.s, uqt1.m.s == ys[length(ys)], NA); 
	lqt0a.m.s	<- replace(lqt0.m.s, lqt0.m.s == ys[sum(AA0.m.s <= 0)], NA); 
	uqt0a.m.s	<- replace(uqt0.m.s, uqt0.m.s == ys[length(ys)], NA); 

	lqt1a.m.s.d	<- replace(lqt1.m.s.d, lqt1.m.s.d == ys[sum(AA1.m.s.d <= 0)], NA); 
	uqt1a.m.s.d	<- replace(uqt1.m.s.d, uqt1.m.s.d == ys[length(ys)], NA); 
	lqt0a.m.s.d	<- replace(lqt0.m.s.d, lqt0.m.s.d == ys[sum(AA0.m.s.d <= 0)], NA); 
	uqt0a.m.s.d	<- replace(uqt0.m.s.d, uqt0.m.s.d == ys[length(ys)], NA); 


	
	# Bounds for quantile effect on treated;

	uqet.m.s<- (qei.s < 0) * (uqt1a.m.s - lqt0a.m.s) + (qei.s >= 0) * replace(uqt1a.s - lqt0a.s, is.na(uqt1a.s - lqt0a.s), 99);
	uqet.m.s<- replace(uqet.m.s, uqet.m.s == 99, NA);
	lqet.m.s<- (qei.s < 0) * replace(lqt1a.s - uqt0a.s, is.na(lqt1a.s - uqt0a.s), -99) + (qei.s >= 0) * (lqt1a.m.s - uqt0a.m.s);
	lqet.m.s<- replace(lqet.m.s, lqet.m.s == -99, NA);


	uqet.m.s.d	<- (qei.s.d < 0) * (uqt1a.m.s.d - lqt0a.m.s.d) + (qei.s.d >= 0) * replace(uqt1a.s.d - lqt0a.s.d, is.na(uqt1a.s.d - lqt0a.s.d), 99);
	uqet.m.s.d	<- replace(uqet.m.s.d, uqet.m.s.d == 99, NA);
	lqet.m.s.d	<- (qei.s.d < 0) * replace(lqt1a.s.d - uqt0a.s.d, is.na(lqt1a.s.d - uqt0a.s.d), -99) + (qei.s.d >= 0) * (lqt1a.m.s.d - uqt0a.m.s.d);
	lqet.m.s.d	<- replace(lqet.m.s.d, lqet.m.s.d == -99, NA);



	# Saving results for joint figure;

	uqet.boots	<- cbind(uqet.boots, uqet.s);
	lqet.boots	<- cbind(lqet.boots, lqet.s);
	qei.boots	<- cbind(qei.boots, qei.s);
	uqetm.boots	<- cbind(uqetm.boots, uqet.m.s);
	lqetm.boots	<- cbind(lqetm.boots, lqet.m.s);

	uqetd.boots	<- cbind(uqetd.boots, uqet.s.d);
	lqetd.boots	<- cbind(lqetd.boots, lqet.s.d);
	qeid.boots	<- cbind(qeid.boots, qei.s.d);
	uqetmd.boots	<- cbind(uqetmd.boots, uqet.m.s.d);
	lqetmd.boots	<- cbind(lqetmd.boots, lqet.m.s.d);

	};

    bootstrap$uqet  	<- mean(scale)*uqet.boots;
    bootstrap$lqet  	<- mean(scale)*lqet.boots;
    bootstrap$qei  	<- mean(scale)*qei.boots;
    bootstrap$uqetm  	<- mean(scale)*uqetm.boots;
    bootstrap$lqetm  	<- mean(scale)*lqetm.boots;

    bootstrap$uqetd  	<- mean(scale)*uqetd.boots;
    bootstrap$lqetd  	<- mean(scale)*lqetd.boots;
    bootstrap$qeid  	<- mean(scale)*qeid.boots;
    bootstrap$uqetmd  	<- mean(scale)*uqetmd.boots;
    bootstrap$lqetmd  	<- mean(scale)*lqetmd.boots;

    return(bootstrap);
    }




#####################################################
# PART II: MAIN PROGRAM
#####################################################


#####################################################
# ESTIMATES
#####################################################

########################
###### T = 8
########################

library(stats);
library(foreign);

# Reading dataset;
data	<- read.dta("data_static_T8_long.dta");
T	<- 8;
n	<- nrow(data)/T;
B	<- 200;
set.seed(888);
attach(data);


# Location and scale time effects;

  meant	<- NULL;
  st   <- NULL;

	for (period in 1:T) {;
		meant[period]	<- mean(y[(tx1*x + (1-tx1)*(1-x)) == 1 & time == period]);
    st[period]    <- sum(y[tx1*x == 1 & time == period] - meant[period]);                  
		};

  scale <- st/st[1];
  loc <-  meant - scale*meant[1];
  
	yte	<- (y - kronecker(matrix(1, nrow = n), loc) ) / kronecker(matrix(1, nrow = n), scale) ;


# Creating indicators and smoothed indicators to estimate cdf;

ys	<- quantile(yte, c(0:1000)/1000);
y_ind.s	<- pnorm(matrix(kronecker(matrix(1, nrow = length(yte), ncol = 1), matrix(ys, ncol = 1)) - kronecker(matrix(yte, ncol = 1), matrix(1, nrow = length(ys), ncol = 1)), nrow = length(yte), ncol = length(ys), byrow = T)/ bw.nrd0(yte)); 

# Creating matrices to estimate bounds of average effects on treated;
 
A1_x1.s    <- NULL;
A0_x1.s		<- NULL;
pA_x1		<- NULL;

for (i in 2:2^T) {;
	pA_x1		<- rbind(pA_x1, mean(tx == i) / mean(tx != 1));
	A1_x1.s		<- rbind(A1_x1.s, apply(kronecker(matrix(x[tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == i], 2, sum) / max(sum(x[tx == i]), 1));
	A0_x1.s		<- rbind(A0_x1.s, apply(kronecker(matrix(1 - x[tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == i], 2, sum) / max(sum(1 - x[tx == i]), 1));
	};

AA1.s	<- t(pA_x1) %*% A1_x1.s; 
AA0.s	<- t(pA_x1) %*% A0_x1.s; 

# Creating matrices to estimate bounds of average effects on treated under monotonicity;

AA1.m.s	<- t(pA_x1) %*% A1_x1.s; 
AA0.m.s	<- t(pA_x1) %*% rbind(A0_x1.s[-(2^T-1),],A1_x1.s[2^T-1,]); 

# Creating matrices to estimate bounds of average effects on first treated;
 
pA_x1.d		<- NULL;
A1_x1.s.d	<- NULL;
A0_x1.s.d	<- NULL;

for (i in 2:2^T) {;
	pA_x1.d		<- rbind(pA_x1.d, mean(tx == i & tx1 == 1) / mean(tx != 1 & tx1 == 1));
	A1_x1.s.d	<- rbind(A1_x1.s.d, apply(kronecker(matrix(xf1[tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == i], 2, sum) / max(sum(xf1[tx == i]), 1));
	A0_x1.s.d	<- rbind(A0_x1.s.d, apply(kronecker(matrix(xf0[tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == i], 2, sum) / max(sum(xf0[tx == i]), 1));
	};

AA1.s.d	<- t(pA_x1.d) %*% A1_x1.s.d; 
AA0.s.d	<- t(pA_x1.d) %*% A0_x1.s.d; 

# Creating matrices to estimate bounds of average effects on first treated under monotonicity;

A0_x1.s.d	<- rbind(A0_x1.s.d[-(2^T-1),], apply(kronecker(matrix(xl1[tx == 2^T], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == 2^T], 2, sum) / max(sum(xl1[tx == 2^T]), 1) );
AA1.m.s.d	<- t(pA_x1.d) %*% A1_x1.s.d; 
AA0.m.s.d	<- t(pA_x1.d) %*% A0_x1.s.d; 

# Creating matrices to estimate identified effects;


pA_xi		<- NULL;
A1_xi.s		<- NULL;
A0_xi.s		<- NULL;
A1_xi.s.d	<- NULL;
A0_xi.s.d	<- NULL;

for (i in 2:(2^T-1)) {;
	pA_xi		<- rbind(pA_xi, mean(tx == i) / mean(tx != 1 & tx != 2^T));
	A1_xi.s		<- rbind(A1_xi.s, apply(kronecker(matrix(x[tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == i], 2, sum) / max(sum(x[tx == i]), 1));
	A0_xi.s		<- rbind(A0_xi.s, apply(kronecker(matrix(1 - x[tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == i], 2, sum) / max(sum(1 - x[tx == i]), 1));
	A1_xi.s.d	<- rbind(A1_xi.s.d, apply(kronecker(matrix(xf1[tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == i], 2, sum) / max(sum(xf1[tx == i]), 1));
	A0_xi.s.d	<- rbind(A0_xi.s.d, apply(kronecker(matrix(xf0[tx == i], ncol = 1), matrix(1, nrow = 1, ncol = length(ys)) ) * y_ind.s[tx == i], 2, sum) / max(sum(xf0[tx == i]), 1));
	};

AAA1.s		<- t(pA_xi) %*% A1_xi.s; 
AAA0.s		<- t(pA_xi) %*% A0_xi.s;
AAA1.s.d	<- t(pA_xi) %*% A1_xi.s.d; 
AAA0.s.d	<- t(pA_xi) %*% A0_xi.s.d;


# Quantile effects;

taus	<- c(1:99)/100;

# Quantile effects without controlling for heterogeneity;

qe	   <- quantile(y[x==1], taus) - quantile(y[x==0], taus);
qe.te  <- quantile(yte[x==1], taus) - quantile(yte[x==0], taus);

# Constructing bounds for quantile effects;

uqt1.s		<- NULL;
lqt1.s		<- NULL;
uqt0.s		<- NULL;
lqt0.s		<- NULL;

qi1.s		<- NULL;
qi0.s		<- NULL;

uqt1.s.d	<- NULL;
lqt1.s.d	<- NULL;
uqt0.s.d	<- NULL;
lqt0.s.d	<- NULL;

qi1.s.d		<- NULL;
qi0.s.d		<- NULL;


for (tau in taus) {;

	lqt1.s	<- c(lqt1.s, ys[sum(AA1.s <= max(0, tau))] );
	uqt1.s	<- c(uqt1.s, ys[sum(AA1.s <= max(0, tau))] );
	lqt0.s	<- c(lqt0.s, ys[max(sum(AA0.s <= max(0, tau - pA_x1[2^T-1,1])), 1)] );
	uqt0.s	<- c(uqt0.s, ys[max(sum(AA0.s <= max(0, tau)), 1)] );

	qi1.s	<- c(qi1.s, ys[sum(AAA1.s <= max(0, tau))] );
	qi0.s	<- c(qi0.s, ys[max(sum(AAA0.s <= max(0, tau)), 1)] );


	lqt1.s.d	<- c(lqt1.s.d, ys[sum(AA1.s.d <= max(0, tau))] );
	uqt1.s.d	<- c(uqt1.s.d, ys[sum(AA1.s.d <= max(0, tau))] );
	lqt0.s.d	<- c(lqt0.s.d, ys[max(sum(AA0.s.d <= max(0, tau - pA_x1.d[2^T-1,1])), 1)] );
	uqt0.s.d	<- c(uqt0.s.d, ys[max(sum(AA0.s.d <= max(0, tau)), 1)] );

	qi1.s.d		<- c(qi1.s.d, ys[sum(AAA1.s.d <= max(0, tau))] );
	qi0.s.d		<- c(qi0.s.d, ys[max(sum(AAA0.s.d <= max(0, tau)), 1)] );

	};



lqt1a.s	<- replace(lqt1.s, lqt1.s == ys[sum(AA1.s <= 0)], NA); 
uqt1a.s	<- replace(uqt1.s, uqt1.s == ys[length(ys)], NA); 
lqt0a.s	<- replace(lqt0.s, lqt0.s == ys[sum(AA0.s <= 0)], NA); 
uqt0a.s	<- replace(uqt0.s, uqt0.s == ys[length(ys)], NA); 

qi1a.s	<- replace(qi1.s, qi1.s == ys[sum(AAA1.s <= 0)], NA); 
qi1a.s	<- replace(qi1.s, qi1.s == ys[length(ys)], NA); 
qi0a.s	<- replace(qi0.s, qi0.s == ys[sum(AAA0.s <= 0)], NA); 
qi0a.s	<- replace(qi0.s, qi0.s == ys[length(ys)], NA); 

lqt0a.s	<- replace(lqt0.s, taus <= pA_x1[2^T-1,1], NA); 
uqt0a.s	<- replace(uqt0.s, taus >= 1 - pA_x1[2^T-1,1], NA); 


lqt1a.s.d	<- replace(lqt1.s.d, lqt1.s.d == ys[sum(AA1.s.d <= 0)], NA); 
uqt1a.s.d	<- replace(uqt1.s.d, uqt1.s.d == ys[length(ys)], NA); 
lqt0a.s.d	<- replace(lqt0.s.d, lqt0.s.d == ys[sum(AA0.s.d <= 0)], NA); 
uqt0a.s.d	<- replace(uqt0.s.d, uqt0.s.d == ys[length(ys)], NA); 

qi1a.s.d	<- replace(qi1.s.d, qi1.s.d == ys[sum(AAA1.s.d <= 0)], NA); 
qi1a.s.d	<- replace(qi1.s.d, qi1.s.d == ys[length(ys)], NA); 
qi0a.s.d	<- replace(qi0.s.d, qi0.s.d == ys[sum(AAA0.s.d <= 0)], NA); 
qi0a.s.d	<- replace(qi0.s.d, qi0.s.d == ys[length(ys)], NA); 

lqt0a.s.d	<- replace(lqt0.s.d, taus <= pA_x1.d[2^T-1,1], NA); 
uqt0a.s.d	<- replace(uqt0.s.d, taus >= 1 - pA_x1.d[2^T-1,1], NA); 

# Bounds for quantile effect on treated;

uqet.s		<- uqt1a.s - lqt0a.s;
lqet.s		<- lqt1a.s - uqt0a.s;

# Bounds for quantile effect on first treated;

uqet.s.d	<- uqt1a.s.d - lqt0a.s.d;
lqet.s.d	<- lqt1a.s.d - uqt0a.s.d;

# Identified effect;

qei.s	<- qi1a.s   - qi0a.s;
qei.s.d	<- qi1a.s.d - qi0a.s.d;

# Constructing bounds for quantile effects imposing monotonicity;

uqt1.m.s	<- NULL;
lqt1.m.s	<- NULL;
uqt0.m.s	<- NULL;
lqt0.m.s	<- NULL;

uqt1.m.s.d	<- NULL;
lqt1.m.s.d	<- NULL;
uqt0.m.s.d	<- NULL;
lqt0.m.s.d	<- NULL;


for (tau in taus) {;

	lqt1.m.s	<- c(lqt1.m.s, ys[sum(AA1.m.s <= max(0, tau))] );
	uqt1.m.s	<- c(uqt1.m.s, ys[sum(AA1.m.s <= max(0, tau))] );
	lqt0.m.s	<- c(lqt0.m.s, ys[max(sum(AA0.m.s <= max(0, tau)), 1)] );
	uqt0.m.s	<- c(uqt0.m.s, ys[max(sum(AA0.m.s <= max(0, tau)), 1)] );

	lqt1.m.s.d	<- c(lqt1.m.s.d, ys[sum(AA1.m.s.d <= max(0, tau))] );
	uqt1.m.s.d	<- c(uqt1.m.s.d, ys[sum(AA1.m.s.d <= max(0, tau))] );
	lqt0.m.s.d	<- c(lqt0.m.s.d, ys[max(sum(AA0.m.s.d <= max(0, tau)), 1)] );
	uqt0.m.s.d	<- c(uqt0.m.s.d, ys[max(sum(AA0.m.s.d <= max(0, tau)), 1)] );

	};


lqt1a.m.s	<- replace(lqt1.m.s, lqt1.m.s == ys[sum(AA1.m.s <= 0)], NA); 
uqt1a.m.s	<- replace(uqt1.m.s, uqt1.m.s == ys[length(ys)], NA); 
lqt0a.m.s	<- replace(lqt0.m.s, lqt0.m.s == ys[sum(AA0.m.s <= 0)], NA); 
uqt0a.m.s	<- replace(uqt0.m.s, uqt0.m.s == ys[length(ys)], NA); 

lqt1a.m.s.d	<- replace(lqt1.m.s.d, lqt1.m.s.d == ys[sum(AA1.m.s.d <= 0)], NA); 
uqt1a.m.s.d	<- replace(uqt1.m.s.d, uqt1.m.s.d == ys[length(ys)], NA); 
lqt0a.m.s.d	<- replace(lqt0.m.s.d, lqt0.m.s.d == ys[sum(AA0.m.s.d <= 0)], NA); 
uqt0a.m.s.d	<- replace(uqt0.m.s.d, uqt0.m.s.d == ys[length(ys)], NA); 

# Bounds for quantile effect on treated;

uqet.m.s	<- (qei.s < 0) * (uqt1a.m.s - lqt0a.m.s) + (qei.s >= 0) * replace(uqt1a.s - lqt0a.s, is.na(uqt1a.s - lqt0a.s), 99);
uqet.m.s	<- replace(uqet.m.s, uqet.m.s == 99, NA);
lqet.m.s	<- (qei.s < 0) * replace(lqt1a.s - uqt0a.s, is.na(lqt1a.s - uqt0a.s), -99) + (qei.s >= 0) * (lqt1a.m.s - uqt0a.m.s);
lqet.m.s	<- replace(lqet.m.s, lqet.m.s == -99, NA);

# Bounds for quantile effect on first treated;

uqet.m.s.d	<- (qei.s.d < 0) * (uqt1a.m.s.d - lqt0a.m.s.d) + (qei.s.d >= 0) * replace(uqt1a.s.d - lqt0a.s.d, is.na(uqt1a.s.d - lqt0a.s.d), 99);
uqet.m.s.d	<- replace(uqet.m.s.d, uqet.m.s.d == 99, NA);
lqet.m.s.d	<- (qei.s.d < 0) * replace(lqt1a.s.d - uqt0a.s.d, is.na(lqt1a.s.d - uqt0a.s.d), -99) + (qei.s.d >= 0) * (lqt1a.m.s.d - uqt0a.m.s.d);
lqet.m.s.d	<- replace(lqet.m.s.d, lqet.m.s.d == -99, NA);

# Rescaling to obtain time-average effect;

uqet.s  	<- mean(scale)*uqet.s;
lqet.s		<- mean(scale)*lqet.s;
uqet.s.d	<- mean(scale)*uqet.s.d;
lqet.s.d	<- mean(scale)*uqet.s.d;
uqet.m.s  <- mean(scale)*uqet.m.s;
lqet.m.s	<- mean(scale)*lqet.m.s;
uqet.m.s.d	<- mean(scale)*uqet.m.s.d;
lqet.m.s.d	<- mean(scale)*uqet.m.s.d;
qei.s	    <- mean(scale)*qei.s;
qei.s.d	  <- mean(scale)*qei.s.d;


# Uniform bands;

rang		<- (taus >= .2 & taus <= .8);
bootstrap 	<- subsamplek(data=data, n=n, B=B, taus, T);
alpha		<- .10;

uqet.b1		<- bootstrap$uqet;
lqet.b1		<- bootstrap$lqet;
qei.b1		<- bootstrap$qei;
uqetm.b1	<- bootstrap$uqetm;
lqetm.b1	<- bootstrap$lqetm;

uqetd.b1	<- bootstrap$uqetd;
lqetd.b1	<- bootstrap$lqetd;
qeid.b1		<- bootstrap$qeid;
uqetmd.b1	<- bootstrap$uqetmd;
lqetmd.b1	<- bootstrap$lqetmd;

rm(bootstrap);

# Static model;

uqet.b		<- uqet.b1[apply(is.na(uqet.b1), 1, max)==0,];
lqet.b		<- lqet.b1[apply(is.na(lqet.b1), 1, max)==0,];
qei.b		<- qei.b1[apply(is.na(qei.b1), 1, max)==0,];
uqetm.b		<- uqetm.b1[apply(is.na(uqetm.b1), 1, max)==0,];
lqetm.b		<- lqetm.b1[apply(is.na(lqetm.b1), 1, max)==0,];

taus.uqet	<- taus[apply(is.na(uqet.b1), 1, max)==0];
taus.lqet	<- taus[apply(is.na(lqet.b1), 1, max)==0];
taus.qei	<- taus[apply(is.na(qei.b1), 1, max)==0];
taus.uqetm	<- taus[apply(is.na(uqetm.b1), 1, max)==0];
taus.lqetm	<- taus[apply(is.na(lqetm.b1), 1, max)==0];

V.uqet   	<- apply(uqet.b, 1, var, na.rm = T);
V.lqet   	<- apply(lqet.b, 1, var, na.rm = T);
V.qei   	<- apply(qei.b, 1, var, na.rm = T);
V.uqetm   	<- apply(uqetm.b, 1, var, na.rm = T);
V.lqetm   	<- apply(lqetm.b, 1, var, na.rm = T);

K.uqet 		<- (uqet.b - matrix(uqet.s[apply(is.na(uqet.b1), 1, max)==0], length(taus.uqet), B, byrow = F)) / matrix(sqrt(V.uqet), length(taus.uqet), B, byrow = F);
K.lqet 		<- (lqet.b - matrix(lqet.s[apply(is.na(lqet.b1), 1, max)==0], length(taus.lqet), B, byrow = F)) / matrix(sqrt(V.lqet), length(taus.lqet), B, byrow = F);
K.qei 		<- abs(qei.b - matrix(qei.s[apply(is.na(qei.b1), 1, max)==0], length(taus.qei), B, byrow = F)) / matrix(sqrt(V.qei), length(taus.qei), B, byrow = F);
K.uqetm		<- (uqetm.b - matrix(uqet.m.s[apply(is.na(uqetm.b1), 1, max)==0], length(taus.uqetm), B, byrow = F)) / matrix(sqrt(V.uqetm), length(taus.uqetm), B, byrow = F);
K.lqetm		<- (lqetm.b - matrix(lqet.m.s[apply(is.na(lqetm.b1), 1, max)==0], length(taus.lqetm), B, byrow = F)) / matrix(sqrt(V.lqetm), length(taus.lqetm), B, byrow = F);



Kmax.uqet 	<- apply(K.uqet[taus.uqet >= .10 & taus.uqet <= .90, ],2, FUN=max, na.rm = T);
Kmax.lqet 	<- apply(K.lqet[taus.lqet >= .10 & taus.lqet <= .90, ],2, FUN=min, na.rm = T);
Kmax.qei 	<- apply(K.qei[taus.qei >= .10 & taus.qei <= .90, ],2,  FUN=max, na.rm = T);
Kmax.uqetm 	<- apply(K.uqetm[taus.uqetm >= .10 & taus.uqetm <= .90, ],2, FUN=max, na.rm = T);
Kmax.lqetm 	<- apply(K.lqetm[taus.lqetm >= .10 & taus.lqetm <= .90, ],2, FUN=min, na.rm = T);

Kalpha.uqet 	<- quantile(Kmax.uqet, 1-alpha);
Kalpha.lqet 	<- quantile(Kmax.lqet, alpha);
Kalpha.qei 	<- quantile(Kmax.qei, 1-alpha);
Kalpha.uqetm 	<- quantile(Kmax.uqetm, 1-alpha);
Kalpha.lqetm 	<- quantile(Kmax.lqetm, alpha);


uqet.se   	<- sqrt(V.uqet);
lqet.se   	<- sqrt(V.lqet);
qei.se   	<- sqrt(V.qei);
uqetm.se   	<- sqrt(V.uqetm);
lqetm.se   	<- sqrt(V.lqetm);


ucb.uqet	<- uqet.s[apply(is.na(uqet.b1), 1, max)==0] + Kalpha.uqet * uqet.se; 
lcb.lqet	<- lqet.s[apply(is.na(lqet.b1), 1, max)==0] + Kalpha.lqet * lqet.se; 
ucb.qei		<- qei.s[apply(is.na(qei.b1), 1, max)==0] + Kalpha.qei * qei.se; 
lcb.qei		<- qei.s[apply(is.na(qei.b1), 1, max)==0] - Kalpha.qei * qei.se; 
ucb.uqetm	<- uqet.m.s[apply(is.na(uqetm.b1), 1, max)==0] + Kalpha.uqetm * uqetm.se; 
lcb.lqetm	<- lqet.m.s[apply(is.na(lqetm.b1), 1, max)==0] + Kalpha.lqetm * lqetm.se; 

#rm(uqet.b1, lqet.b1, qei.b1, uqetm.b1, lqetm.b1);
#rm(uqet.b, lqet.b, qei.b, uqetm.b, lqetm.b);

# Dynamic model;

uqetd.b		<- uqetd.b1[apply(is.na(uqetd.b1), 1, max)==0,];
lqetd.b		<- lqetd.b1[apply(is.na(lqetd.b1), 1, max)==0,];
qeid.b		<- qeid.b1[apply(is.na(qeid.b1), 1, max)==0,];
uqetmd.b	<- uqetmd.b1[apply(is.na(uqetmd.b1), 1, max)==0,];
lqetmd.b	<- lqetmd.b1[apply(is.na(lqetmd.b1), 1, max)==0,];

taus.uqetd	<- taus[apply(is.na(uqetd.b1), 1, max)==0];
taus.lqetd	<- taus[apply(is.na(lqetd.b1), 1, max)==0];
taus.qeid	<- taus[apply(is.na(qeid.b1), 1, max)==0];
taus.uqetmd	<- taus[apply(is.na(uqetmd.b1), 1, max)==0];
taus.lqetmd	<- taus[apply(is.na(lqetmd.b1), 1, max)==0];

V.uqetd   	<- apply(uqetd.b, 1, var, na.rm = T);
V.lqetd   	<- apply(lqetd.b, 1, var, na.rm = T);
V.qeid   	<- apply(qeid.b, 1, var, na.rm = T);
V.uqetmd   	<- apply(uqetmd.b, 1, var, na.rm = T);
V.lqetmd   	<- apply(lqetmd.b, 1, var, na.rm = T);

K.uqetd 	<- (uqetd.b - matrix(uqet.s.d[apply(is.na(uqetd.b1), 1, max)==0], length(taus.uqetd), B, byrow = F)) / matrix(sqrt(V.uqetd), length(taus.uqetd), B, byrow = F);
K.lqetd 	<- (lqetd.b - matrix(lqet.s.d[apply(is.na(lqetd.b1), 1, max)==0], length(taus.lqetd), B, byrow = F)) / matrix(sqrt(V.lqetd), length(taus.lqetd), B, byrow = F);
K.qeid 		<- abs(qeid.b - matrix(qei.s.d[apply(is.na(qeid.b1), 1, max)==0], length(taus.qeid), B, byrow = F)) / matrix(sqrt(V.qeid), length(taus.qeid), B, byrow = F);
K.uqetmd	<- (uqetmd.b - matrix(uqet.m.s.d[apply(is.na(uqetmd.b1), 1, max)==0], length(taus.uqetmd), B, byrow = F)) / matrix(sqrt(V.uqetmd), length(taus.uqetmd), B, byrow = F);
K.lqetmd	<- (lqetmd.b - matrix(lqet.m.s.d[apply(is.na(lqetmd.b1), 1, max)==0], length(taus.lqetmd), B, byrow = F)) / matrix(sqrt(V.lqetmd), length(taus.lqetmd), B, byrow = F);



Kmax.uqetd 	<- apply(K.uqetd[taus.uqetd >= .10 & taus.uqetd <= .90, ],2, FUN=max, na.rm = T);
Kmax.lqetd 	<- apply(K.lqetd[taus.lqetd >= .10 & taus.lqetd <= .90, ],2, FUN=min, na.rm = T);
Kmax.qeid 	<- apply(K.qeid[taus.qeid >= .10 & taus.qeid <= .90, ],2,  FUN=max, na.rm = T);
Kmax.uqetmd 	<- apply(K.uqetmd[taus.uqetmd >= .10 & taus.uqetmd <= .90, ],2, FUN=max, na.rm = T);
Kmax.lqetmd 	<- apply(K.lqetmd[taus.lqetmd >= .10 & taus.lqetmd <= .90, ],2, FUN=min, na.rm = T);

Kalpha.uqetd 	<- quantile(Kmax.uqetd, 1-alpha);
Kalpha.lqetd 	<- quantile(Kmax.lqetd, alpha);
Kalpha.qeid 	<- quantile(Kmax.qeid, 1-alpha);
Kalpha.uqetmd 	<- quantile(Kmax.uqetmd, 1-alpha);
Kalpha.lqetmd 	<- quantile(Kmax.lqetmd, alpha);


uqetd.se   	<- sqrt(V.uqetd);
lqetd.se   	<- sqrt(V.lqetd);
qeid.se   	<- sqrt(V.qeid);
uqetmd.se   	<- sqrt(V.uqetmd);
lqetmd.se   	<- sqrt(V.lqetmd);


ucb.uqetd	<- uqet.s.d[apply(is.na(uqetd.b1), 1, max)==0] + Kalpha.uqetd * uqetd.se; 
lcb.lqetd	<- lqet.s.d[apply(is.na(lqetd.b1), 1, max)==0] + Kalpha.lqetd * lqetd.se; 
ucb.qeid	<- qei.s.d[apply(is.na(qeid.b1), 1, max)==0]   + Kalpha.qeid * qeid.se; 
lcb.qeid	<- qei.s.d[apply(is.na(qeid.b1), 1, max)==0]   - Kalpha.qeid * qeid.se; 
ucb.uqetmd	<- uqet.m.s.d[apply(is.na(uqetmd.b1), 1, max)==0] + Kalpha.uqetmd * uqetmd.se; 
lcb.lqetmd	<- lqet.m.s.d[apply(is.na(lqetmd.b1), 1, max)==0] + Kalpha.lqetmd * lqetmd.se; 

#rm(uqetd.b1, lqetd.b1, qeid.b1, uqetmd.b1, lqetmd.b1);
#rm(uqetd.b, lqetd.b, qeid.b, uqetmd.b, lqetmd.b);

# Saving results for joint figure;

ucb.uqet.t8	<- ucb.uqet;
lcb.lqet.t8	<- lcb.lqet;
ucb.qei.t8	<- ucb.qei;
lcb.qei.t8	<- lcb.qei;
ucb.uqetm.t8	<- ucb.uqetm;
lcb.lqetm.t8	<- lcb.lqetm;

taus.uqet.t8	<- taus.uqet;
taus.lqet.t8	<- taus.lqet;
taus.qei.t8	<- taus.qei;
taus.uqetm.t8	<- taus.uqetm;
taus.lqetm.t8	<- taus.lqetm;

ucb.uqetd.t8	<- ucb.uqetd;
lcb.lqetd.t8	<- lcb.lqetd;
ucb.qeid.t8	<- ucb.qeid;
lcb.qeid.t8	<- lcb.qeid;
ucb.uqetmd.t8	<- ucb.uqetmd;
lcb.lqetmd.t8	<- lcb.lqetmd;

taus.uqetd.t8	<- taus.uqetd;
taus.lqetd.t8	<- taus.lqetd;
taus.qeid.t8	<- taus.qeid;
taus.uqetmd.t8	<- taus.uqetmd;
taus.lqetmd.t8	<- taus.lqetmd;

detach(data);
rm(data);

rang	<- (taus >= .10 & taus <= .90);
rang	<- (taus >= .1 & taus <= .9);



postscript("identified_effects_te.eps",horizontal=FALSE,onefile=FALSE,height=7,width=6,pointsize=10);

plot(c(min(taus[rang]), max(taus[rang])), c(-.2, .5), type="n", xlab= expression(lambda), ylab = "Difference of log-wages", main="");
title(main = "B - Static model with time effects: identified effect", cex.main=1);

  lines(taus[rang], qei.s[rang], lty = 1, col = 4);
  lines(taus.qei.t8[taus.qei.t8 >= .10 & taus.qei.t8 <= .90], ucb.qei.t8[taus.qei.t8 >= .10 & taus.qei.t8 <= .90], col = 4, lty = 2)
	lines(taus.qei.t8[taus.qei.t8 >= .10 & taus.qei.t8 <= .90], lcb.qei.t8[taus.qei.t8 >= .10 & taus.qei.t8 <= .90], col = 4, lty = 2)
  lines(taus[rang], qe.te[rang], lty = 3, col = 1, lwd = 2);
	abline(h = 0, lty = 3);

	legend(min(taus[rang]), 0.5, c("With heterogeneity", "90% CI", "Without heterogeneity"), lwd = c(1,1,2), lty = c(1,2,3), bty="n",col=c(4,4,1));


dev.off();

postscript("bounds_static_te.eps",horizontal=FALSE,onefile=FALSE,height=7,width=6,pointsize=10);

plot(c(min(taus[rang]), max(taus[rang])), c(-.5, 1), type="n", xlab= expression(lambda), ylab = "Difference of log-wages", main="");
title(main = "C - Static model with time effects: effect on ever unionized", cex.main=1);

  lines(taus[rang], uqet.s[rang]);
	lines(taus[rang], lqet.s[rang]);
	lines(taus[rang], lqet.m.s[rang], col = 2);
	abline(h = 0, lty = 3);
	lines(taus.uqet.t8[taus.uqet.t8 >= .10 & taus.uqet.t8 <= .90], ucb.uqet.t8[taus.uqet.t8 >= .10 & taus.uqet.t8 <= .90], col = 1, lty = 2)
	lines(taus.lqet.t8[taus.lqet.t8 >= .10 & taus.lqet.t8 <= .90], lcb.lqet.t8[taus.lqet.t8 >= .10 & taus.lqet.t8 <= .90], col = 1, lty = 2)
	lines(taus.lqetm.t8[taus.lqetm.t8 >= .10 & taus.lqetm.t8 <= .90], lcb.lqetm.t8[taus.lqetm.t8 >= .10 & taus.lqetm.t8 <= .90], col = 2, lty = 2)

	legend(min(taus[rang]), 1, c("Bound", "90% CI", "Monotonicity", "90% CI"), lwd = c(1,1,1,1), lty = c(1,2,1,2), bty="n",col=c(1,1,2,2));


dev.off();

