###########################################################################################################################
## Chernozhukov, Fernandez-Val, Hahn, and Newey: "Average and Quantile Effects in Nonseparable Panel Models"
## R program to generate Logit column in Table 3
###########################################################################################################################

# DATA: NLSY 1990-92-94 (1.587 MARRIED WOMEN) 

# Variables:
#    1 - y: LFP
#    2 - x: KIDS

# This version: 01/11/2014 (Correction to the code posted at publication)
# Thanks to Mian Huang for pointing out some discrepancies between the code initially posted in the on-line appendix for the perturbed bootstrap and the
# description of this method in the article


#####################################################
# PART I: PRELIMINARY FUNCTIONS
#####################################################

rm(list = ls());

## PAY calculates the true probabilities 

PAY <- function(x, y, x0, T)
	{
	pAY	<- matrix(0, nrow = 2^T, ncol =1);
	if (T == 2) {;
		pAY[1,1]	<- sum((y == 1) * (x == x0)) / sum((x == x0));
		pAY[2,1]	<- sum((y == 2) * (x == x0)) / sum((x == x0));
		pAY[3,1]	<- sum((y == 3) * (x == x0)) / sum((x == x0));
		pAY[4,1]	<- sum((y == 4) * (x == x0)) / sum((x == x0));
		};
	if (T == 3) {;
		pAY[1,1]	<- sum((y == 1) * (x == x0)) / sum((x == x0));
		pAY[2,1]	<- sum((y == 2) * (x == x0)) / sum((x == x0));
		pAY[3,1]	<- sum((y == 3) * (x == x0)) / sum((x == x0));
		pAY[4,1]	<- sum((y == 4) * (x == x0)) / sum((x == x0));
		pAY[5,1]	<- sum((y == 5) * (x == x0)) / sum((x == x0));
		pAY[6,1]	<- sum((y == 6) * (x == x0)) / sum((x == x0));
		pAY[7,1]	<- sum((y == 7) * (x == x0)) / sum((x == x0));
		pAY[8,1]	<- sum((y == 8) * (x == x0)) / sum((x == x0));
		};
	return(pAY);
	};



## PAYm calculates the true probabilities using the model 

PAYm <- function(AY, x, gamma, alphas, rhos)
	{
	nAY   <- nrow(AY);
	T     <- ncol(AY);
	pAY   <- matrix(0,nAY,1);
	M     <- length(alphas);
	for (i in 1:nAY) {;
		aux <- matrix(0,M,T);
		for (t in 1:T) {;
			aux[,t] <- (plogis(gamma*x[1,t]  + alphas, location = 0, scale = sqrt(3)/pi))^AY[i,t] * (1 - plogis(gamma*x[1,t]  + alphas, location = 0, scale = sqrt(3)/pi))^(1 - AY[i,t]);
			};
		pAY[i,1] <- sum(rhos * apply(aux,1,prod));
		};
	return(pAY);
	};


## PIA calculates the conditional probabilities estimated by NPMLE and ANPMLE 

PIAY <- function(AY, x, gamma, alphas)
	{
	nAX   <- nrow(AX);
	nAY   <- nrow(AY);
	T     <- ncol(AY);
	M     <- length(alphas);
	PIAY  <- matrix(0,nAY, M);
	for (i in 1:nAY) {;
                aux <- matrix(0,M,T);
		for (t in 1:T) {;
			aux[,t] <- (plogis(gamma*x[1,t]  + alphas, location = 0, scale = sqrt(3)/pi))^AY[i,t] * (1 - plogis(gamma*x[1,t] + alphas, location = 0, scale = sqrt(3)/pi))^(1 - AY[i,t]);
			};
		PIAY[i, ] <- apply(aux,1,prod);
		};
	return(PIAY);
	};


## ME calculates individual marginal effects

ME <- function(gamma, alphas, x)
	
	{
	T	<- ncol(x);
	return(plogis(gamma + alphas, location = 0, scale = sqrt(3)/pi) - plogis(alphas, location = 0, scale = sqrt(3)/pi));
	};



## Determines the average number of values with positive probability in the estimated distriution for the alphas

support <- function(prhos_x)
	{
	return(mean(apply(prhos_x>0,2,sum)));
	}
	
	


#####################################################
# PART II: MAIN PROGRAM
#####################################################

######################################################################################################################################
######                                                      T = 2
######################################################################################################################################


library(stats);
library(lpSolve);
library(quadprog);
library(foreign);

#############################################;
# POINT ESTIMATES AND CANONICAL PROJECTION;
#############################################;

# NOTE: UNCOMMENT LINES 335 AND 336 TO OBTAIN CANONICAL PROJECTION UPPER AND LOWER END-POINTS OF CI'S FOR ATE

# Reading dataset;
data	<- read.dta("data_static_T2.dta");
n	<- nrow(data);
attach(data);

# Parameters of the problem

TT      <- 2;
alpha0s <- c(-Inf,4*(-10:10)/10,Inf);
alphas 	<- c(-Inf,(-80:80)/10,Inf);
M      	<- length(alphas);
M0	<- length(alpha0s);

A      <- t(cbind(c(0,0), c(0,1), c(1,0), c(1,1)));	
AY     <- A;
AX     <- A;

pAX	    <- matrix(0, nrow = 2^TT, ncol = 1);
pAX[1,1]    <- mean((x==1));
pAX[2,1]    <- mean((x==2));
pAX[3,1]    <- mean((x==3));
pAX[4,1]    <- mean((x==4));

# Parameters of the optimization

gridgamma0 <- 1;
gridgamma <- 201;
ugamma    <- gridgamma - 1;
gammas    <- (-ugamma:0)/(gridgamma-1);
gammas	  <- c(-100:20)/100;
gridgamma <- length(gammas);

# Initialization of variables

OBJ    <- matrix(-999, gridgamma0, gridgamma);
OBJ2   <- matrix(-999, gridgamma0, gridgamma);

meffs_npmle    <- matrix(-999, gridgamma0, gridgamma);

umeffs_x00_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x01_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x10_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x11_npmle <- matrix(-999, gridgamma0, gridgamma);

lmeffs_x00_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x01_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x10_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x11_npmle <- matrix(999, gridgamma0, gridgamma);

support_alpha_npmle   <- matrix(-999, gridgamma0, gridgamma);
usupport_alpha_npmle  <- matrix(-999, gridgamma0, gridgamma);
lsupport_alpha_npmle  <- matrix(-999, gridgamma0, gridgamma);


nAY  <- nrow(AY);
nAX  <- nrow(AX);
T    <- ncol(AX);

f.obj <- t(cbind(t(rep(1,nAY*nAX+nAX)),t(rep(0,nAX*M))));
f.dir <- t(cbind(t(rep("=",nAY*nAX+nAX)),t(rep(">=", nAX*(nAY+1+M)))));
f_x.dir <- t(cbind(t(rep("=",nAY+1)),t(rep(">=", M))));


lnosol <- 0;
unosol <- 0;

lnosol_x00 <- 0;
unosol_x00 <- 0;
lnosol_x11 <- 0;
unosol_x11 <- 0;

i <- 1;

# Observed conditional probabilities;

	pAY_x00     <- PAY(x, y, 1, TT);
	pAY_x01     <- PAY(x, y, 2, TT);
	pAY_x10     <- PAY(x, y, 3, TT);
	pAY_x11     <- PAY(x, y, 4, TT);


#################
# NPMLE
#################

###############################;
## RIDGE QUADRATIC PROGRAMMING;
###############################;

MM   <- M0;
lambda	<- 1/(n*log(n));

alpha0s_x00	<- alpha0s;
alpha0s_x01	<- alpha0s;
alpha0s_x10	<- alpha0s;
alpha0s_x11	<- alpha0s;

for (j in 1:gridgamma) {;

	gamma <- gammas[j];


	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma, alpha0s);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma, alpha0s);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma, alpha0s);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma, alpha0s);

	MM_x00	   <- length(alpha0s);
	MM_x01	   <- length(alpha0s);
	MM_x10	   <- length(alpha0s);
	MM_x11	   <- length(alpha0s);

# 1st iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11 ));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00), (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01))), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma, alpha0s_x11, as.vector(prhos_x11));


# 2nd iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]   <- 2*n*QP$value;

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma, alpha0s_x11, as.vector(prhos_x11));


# Final iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]   <- 2*n*QP$value;

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);
	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma, alpha0s_x11, as.vector(prhos_x11));
                        
  OBJ2[i,j]  <- n * t(rbind( (pAY_x00-ppAY_x00) / sqrt(ppAY_x00), (pAY_x01-ppAY_x01) / sqrt(ppAY_x01), (pAY_x10-ppAY_x10) / sqrt(ppAY_x10), (pAY_x11-ppAY_x11) / sqrt(ppAY_x11) ) ) %*% kronecker(diag(as.vector(pAX)), diag(1, nrow = nAX)) %*% rbind((pAY_x00-ppAY_x00)/sqrt(ppAY_x00), (pAY_x01-ppAY_x01)/sqrt(ppAY_x01), (pAY_x10-ppAY_x10)/sqrt(ppAY_x10), (pAY_x11-ppAY_x11)/sqrt(ppAY_x11)) ;
                        
	};

	gamma0	  <- gammas[rank(OBJ) == 1];
  gammas0.cp <- gammas[OBJ2 <= qchisq(.95, nAX*(nAY-1))];
#  gamma0    <- min(gammas0.cp); # Uncomment to obtain lower end-point of canonical project CI for ATE;
#  gamma0    <- max(gammas0.cp); # Uncomment to obtain upper end-point of canonical project CI for ATE;

	pAY_x00     <- PAY(x, y, 1, TT);
	pAY_x01     <- PAY(x, y, 2, TT);
	pAY_x10     <- PAY(x, y, 3, TT);
	pAY_x11     <- PAY(x, y, 4, TT);

	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma0, alpha0s_x00);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma0, alpha0s_x01);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma0, alpha0s_x10);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma0, alpha0s_x11);

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11 ));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00), (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01))), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));


	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));


	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));

 	pAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	pAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	pAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	pAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));



# LINEAR PROGRAMMING;

	# Optimization LP problem
		# Solves max (f.obj)'x
		#         st (f.con) x "f.dir" f.rhs
        	# where x is the vector of unknowns
        	#       f.obj is the vector of coefficients in the objective function
        	#       f.con is the matrix of coefficients of the linear constraints
        	#       f.dir is the vector of directions of the constraints
        	#       f.rhs is the vector of coefficients in the rhs of the constraints

	## MODEL PARAMETERS
	# In this example for the model parameters the vector of unknowns is sorted (in the notation of the paper)
		# x = (w_{1}, ..., w_{nAX}, v_{1,1}, ..., v_{nAY,nAX}, z_{1}, ..., z_{nAX*M})   nAX *(1 + nAY + M) unknows
		# Constraints and objective function: 
		# Min_{w,v,z} sum_{j} w_{j} + sum_{i,j} v_{i,j}
		# s.t  v_{i,j} + sum_{m} z_{(j-1)*M + m} pi(A_{i} | x^{j}, \gamma) = P(A_{i}| x^{j}, \gamma_{0})    nAY*nAX constraints
		#      w_{j} + sum_{m} z_{(j-1)*M + m} = 1                                                          nAX constraints
		#      v_{i,j}, w_{j}, z_{m} >= 0                                                                   nAX *(1 + nAY + M) constraints

	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma0, alphas);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma0, alphas);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma0, alphas);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma0, alphas);

	a1_x00    <- cbind(matrix(0, nAY, nAX), diag(rep(1,nAY)), matrix(0, nAY, 3*nAY), piAY_x00, matrix(0, nAY, 3*M));
	a1_x01    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, nAY), diag(rep(1,nAY)), matrix(0, nAY, 2*nAY),  matrix(0, nAY, M), piAY_x01, matrix(0, nAY, 2*M));
	a1_x10    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 2*nAY), diag(rep(1,nAY)), matrix(0, nAY, nAY),  matrix(0, nAY, 2*M), piAY_x10, matrix(0, nAY, M));
	a1_x11    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 3*nAY), diag(rep(1,nAY)), matrix(0, nAY, 3*M), piAY_x11);
	a2_x00    <- cbind(matrix(c(1,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(1, M)),  t(rep(0, 3*M)));
	a2_x01    <- cbind(matrix(c(0,1,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, M)), t(rep(1, M)),  t(rep(0, 2*M)));
	a2_x10    <- cbind(matrix(c(0,0,1,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 2*M)), t(rep(1, M)),  t(rep(0, M)));
	a2_x11    <- cbind(matrix(c(0,0,0,1),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 3*M)), t(rep(1, M)));
        a3        <- diag(rep(1, nAX*(1 + nAY + M)));
        f.con     <- rbind(a1_x00, a1_x01,a1_x10,a1_x11, a2_x00, a2_x01, a2_x10, a2_x11, a3);
	f.rhs     <- t(cbind(t(pAY_x00), t(pAY_x01),t(pAY_x10),t(pAY_x11),t(rep(1, nAX)), t(rep(0, nAX*(1+nAY+M)))));

	LP        <- lp("min", f.obj, f.con, f.dir, f.rhs);


	gamma     <- gamma0;


	## INDIVIDUAL MARGINAL EFFECTS
	# Using the same notation as for model parameters
		# x = (z_{1}, ..., z_{M})   M unknows
	# Constraints and objective function: 
		# Min/Max_{z} sum_{j}  sum_{m} z_{m} me(a_{m}|gamma,x^{j} | X_{1}, ..., X_{T})
		# s.t  sum_{m} z_{m} pi(A_{i} | x^{j}, \gamma) = P(A_{i}| x^{j}, \gamma_{0})    nAY constraints
		#      sum_{m} z_{m} = 1                                                        1 constraints
		#      z_{m} >= 0								M constraints




		meffs_x00    <- ME(gamma, alphas, t(AX[1,]));
		meffs_x01    <- ME(gamma, alphas, t(AX[2,]));
		meffs_x10    <- ME(gamma, alphas, t(AX[3,]));
		meffs_x11    <- ME(gamma, alphas, t(AX[4,]));


		## x = (0,0);

			fmeff_x00.obj   <- meffs_x00;
			fmeff_x00.dir   <- f_x.dir;
			fmeff_x00.rhs  	<- t(cbind(t(pAY_x00),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x00.con	<- rbind(piAY_x00, t(rep(1, M)), diag(1, M));

			LPlmeff_x00     <- lp("min", fmeff_x00.obj, fmeff_x00.con, fmeff_x00.dir, fmeff_x00.rhs);
			lprhos_x00      <- matrix(LPlmeff_x00$solution, ncol = 1);

			if (LPlmeff_x00$status == 0) {; 
				lmeffs_x00_npmle[i,j]	<- LPlmeff_x00$objval;
				};
				
			if (LPlmeff_x00$status == 2) {;
				lmeffs_x00_npmle[i,j]	<- NaN;
				lnosol_x00             	<- lnosol_x00 + 1;
				};

			LPumeff_x00   <- lp("max", fmeff_x00.obj, fmeff_x00.con, fmeff_x00.dir, fmeff_x00.rhs);
			uprhos_x00    <- matrix(LPumeff_x00$solution, ncol = 1);
			if (LPumeff_x00$status == 0) {; 
				umeffs_x00_npmle[i,j]	<- LPumeff_x00$objval;
				};
				
			if (LPumeff_x00$status == 2) {;
				umeffs_x00_npmle[i,j]	<- NaN;
				unosol_x00             	<- unosol_x00 + 1;
				};


      ## x = (0,1);

      lmeffs_x01_npmle[i,j]	<- pAY_x01[2,1] - pAY_x01[3,1];
      umeffs_x01_npmle[i,j]	<- pAY_x01[2,1] - pAY_x01[3,1];

      ## x = (1,0);

      lmeffs_x10_npmle[i,j]	<- pAY_x10[3,1] - pAY_x10[2,1];
      umeffs_x10_npmle[i,j]	<- pAY_x10[3,1] - pAY_x10[2,1];


		## x = (1,1);

			fmeff_x11.obj   <- meffs_x11;
			fmeff_x11.dir   <- f_x.dir;
			fmeff_x11.rhs  	<- t(cbind(t(pAY_x11),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x11.con	<- rbind(piAY_x11, t(rep(1, M)), diag(1, M));

			LPlmeff_x11     <- lp("min", fmeff_x11.obj, fmeff_x11.con, fmeff_x11.dir, fmeff_x11.rhs);
			lprhos_x11      <- matrix(LPlmeff_x11$solution, ncol = 1);

			if (LPlmeff_x11$status == 0) {; 
				lmeffs_x11_npmle[i,j]	<- LPlmeff_x11$objval;
				};
				
			if (LPlmeff_x11$status == 2) {;
				lmeffs_x11_npmle[i,j]	<- NaN;
				lnosol_x11             	<- lnosol_x11 + 1;
				};

			LPumeff_x11   <- lp("max", fmeff_x11.obj, fmeff_x11.con, fmeff_x11.dir, fmeff_x11.rhs);
			uprhos_x11    <- matrix(LPumeff_x11$solution, ncol = 1);
			if (LPumeff_x11$status == 0) {; 
				umeffs_x11_npmle[i,j]	<- LPumeff_x11$objval;
				};
				
			if (LPumeff_x11$status == 2) {;
				umeffs_x11_npmle[i,j]	<- NaN;
				unosol_x11             	<- unosol_x11 + 1;
				};


	umeff_x00_npmle  <- apply(umeffs_x00_npmle,1,max);
	umeff_x01_npmle  <- apply(umeffs_x01_npmle,1,max);
	umeff_x10_npmle  <- apply(umeffs_x10_npmle,1,max);
	umeff_x11_npmle  <- apply(umeffs_x11_npmle,1,max);
	umeff_npmle	 <- pAX[1,1]*umeff_x00_npmle + pAX[2,1]*umeff_x01_npmle + pAX[3,1]*umeff_x10_npmle + pAX[4,1]*umeff_x11_npmle;

	lmeff_x00_npmle  <- apply(lmeffs_x00_npmle,1,min);
	lmeff_x01_npmle  <- apply(lmeffs_x01_npmle,1,min);
	lmeff_x10_npmle  <- apply(lmeffs_x10_npmle,1,min);
	lmeff_x11_npmle  <- apply(lmeffs_x11_npmle,1,min);
	lmeff_npmle	 <- pAX[1,1]*lmeff_x00_npmle + pAX[2,1]*lmeff_x01_npmle + pAX[3,1]*lmeff_x10_npmle + pAX[4,1]*lmeff_x11_npmle;

# Coefficient SP-model;
print(gamma0);

# AME SP-bounds;
print(lmeff_npmle);
print(umeff_npmle);

# Canonical projection: Coefficient;
print(min(gammas0.cp));
print(max(gammas0.cp));

detach(data);



##########################;
### PERTURBED BOOTSTRAP;
##########################;


library(stats);
library(lpSolve);
library(quadprog);
library(foreign);

# Reading dataset;
data	<- read.dta("data_static_T2.dta");
n	<- nrow(data);
attach(data);

# Parameters of the problem

TT      <- 2;
alpha0s <- c(-Inf,4*(-10:10)/10,Inf);
alphas 	<- c(-Inf,(-80:80)/10,Inf);
M      	<- length(alphas);
M0	<- length(alpha0s);

A      <- t(cbind(c(0,0), c(0,1), c(1,0), c(1,1)));	
AY     <- A;
AX     <- A;

pAX	    <- matrix(0, nrow = 2^TT, ncol = 1);
pAX[1,1]    <- mean((x==1));
pAX[2,1]    <- mean((x==2));
pAX[3,1]    <- mean((x==3));
pAX[4,1]    <- mean((x==4));

nAX	    <- 2^TT;
nAY	    <- 2^TT;



# Parameters of the optimization

gridgamma0 <- 1;

set.seed(888);   

nM		<- 100;
nB	 	<- 200;
beta		<- .01;
alpha		<- .05;

# Initialization of variables

gamma.m		<- NULL;
lgamma.m	<- NULL;
ugamma.m	<- NULL;

umeff_x00.m	<- NULL;
lmeff_x00.m	<- NULL;
umeff_x01.m	<- NULL;
lmeff_x01.m	<- NULL;
umeff_x10.m	<- NULL;
lmeff_x10.m	<- NULL;
umeff_x11.m	<- NULL;
lmeff_x11.m	<- NULL;

lmeff.m		<- NULL;
umeff.m		<- NULL;

gammas	  <- c(-150:50)/100;
gridgamma <- length(gammas);
f_x.dir <- t(cbind(t(rep("=",nAY+1)),t(rep(">=", M))));


pAX0		<- pAX;
reject		<- 0;


for (m in 1:nM) {;

# 1 - PERTURBATION

chisqtest	<- 0;

while (chisqtest == 0) {;
	pAX1		<- pAX0;	# Fixes probabilities of X to the observed probabilities in the dataset
	nnAX1		<- n*pAX1;
	pAY1_x00	<- rmultinom(1, size = nnAX1[1,1], prob = PAY(x,y,1,TT))/nnAX1[1,1];
	pAY1_x01	<- rmultinom(1, size = nnAX1[2,1], prob = PAY(x,y,2,TT))/nnAX1[2,1];
	pAY1_x10	<- rmultinom(1, size = nnAX1[3,1], prob = PAY(x,y,3,TT))/nnAX1[3,1];
	pAY1_x11	<- rmultinom(1, size = nnAX1[4,1], prob = PAY(x,y,4,TT))/nnAX1[4,1];
	chisqstat	<- n * ( sum( (pAX1[1,1] * pAY1_x00 - pAX0[1,1] * PAY(x,y,1,TT))^2 / (pAX1[1,1] * pAY1_x00)) +  sum( (pAX1[2,1] * pAY1_x01 - pAX0[2,1] * PAY(x,y,2,TT))^2 / (pAX1[2,1] * pAY1_x01)) +  sum( (pAX1[3,1] * pAY1_x10 - pAX0[3,1] * PAY(x,y,3,TT))^2 / (pAX1[3,1] * pAY1_x10)) +  sum( (pAX1[4,1] * pAY1_x11 - pAX0[4,1] * PAY(x,y,4,TT))^2 / (pAX1[4,1] * pAY1_x11)) );
	if (chisqstat < qchisq(1-beta, nAX*(nAY-1))) chisqtest <- 1;
	if (chisqstat >= qchisq(1-beta, nAX*(nAY-1))) reject <- reject + 1;
	};

# 2 - BOOTSTRAP

gamma.b		<- NULL;
lgamma.b	<- NULL;
ugamma.b	<- NULL;

umeff_x00.b	<- NULL;
lmeff_x00.b	<- NULL;
umeff_x01.b	<- NULL;
lmeff_x01.b	<- NULL;
umeff_x10.b	<- NULL;
lmeff_x10.b	<- NULL;
umeff_x11.b	<- NULL;
lmeff_x11.b	<- NULL;

lmeff.b		<- NULL;
umeff.b		<- NULL;


for (b in 1:nB) {;
	
	pAX	<- pAX0;	# Fixes probabilities of X to the observed probabilities in the dataset
	nnAX	<- n*pAX;
	pAY_x00	<- rmultinom(1, size = nnAX[1,1], prob = pAY1_x00)/nnAX[1,1];
	pAY_x01	<- rmultinom(1, size = nnAX[2,1], prob = pAY1_x01)/nnAX[2,1];
	pAY_x10	<- rmultinom(1, size = nnAX[3,1], prob = pAY1_x10)/nnAX[3,1];
	pAY_x11	<- rmultinom(1, size = nnAX[4,1], prob = pAY1_x11)/nnAX[4,1];

	OBJ    <- matrix(999, gridgamma0, gridgamma);

	umeffs_x00_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x01_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x10_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x11_npmle <- matrix(-999, gridgamma0, gridgamma);

	lmeffs_x00_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x01_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x10_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x11_npmle <- matrix(999, gridgamma0, gridgamma);

	i	  <- 1;

###############################;
## RIDGE QUADRATIC PROGRAMMING;
###############################;

MM   <- M0;
lambda	<- 1/sqrt(n);
lambda	<- 1/(n*log(n));

alpha0s_x00	<- alpha0s;
alpha0s_x01	<- alpha0s;
alpha0s_x10	<- alpha0s;
alpha0s_x11	<- alpha0s;

for (j in 1:gridgamma) {;

	gamma <- gammas[j];


	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma, alpha0s);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma, alpha0s);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma, alpha0s);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma, alpha0s);

	MM_x00	   <- length(alpha0s);
	MM_x01	   <- length(alpha0s);
	MM_x10	   <- length(alpha0s);
	MM_x11	   <- length(alpha0s);

# 1st iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11 ));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00), (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01))), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma, alpha0s_x11, as.vector(prhos_x11));


# 2nd iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]   <- 2*n*QP$value;

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma, alpha0s_x11, as.vector(prhos_x11));


# Final iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]   <- 2*n*QP$value;


	};

	gamma0	  <- gammas[rank(OBJ) == 1];

	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma0, alpha0s_x00);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma0, alpha0s_x01);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma0, alpha0s_x10);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma0, alpha0s_x11);

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11 ));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00), (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01))), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));


	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));


	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	pAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	pAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	pAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	pAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));


	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma0, alphas);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma0, alphas);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma0, alphas);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma0, alphas);

	gamma     <- gamma0;


	## INDIVIDUAL MARGINAL EFFECTS

	# Using the same notation as for model parameters
		# x = (z_{1}, ..., z_{M})   M unknows
	# Constraints and objective function: 
		# Min/Max_{z} sum_{j}  sum_{m} z_{m} me(a_{m}|gamma,x^{j} | X_{1}, ..., X_{T})
		# s.t  sum_{m} z_{m} pi(A_{i} | x^{j}, \gamma) = P(A_{i}| x^{j}, \gamma_{0})    nAY constraints
		#      sum_{m} z_{m} = 1                                                        1 constraints
		#      z_{m} >= 0								M constraints


	J	<- 1;

	meffs_x00    <- ME(gamma, alphas, t(AX[1,]));
	meffs_x01    <- ME(gamma, alphas, t(AX[2,]));
	meffs_x10    <- ME(gamma, alphas, t(AX[3,]));
	meffs_x11    <- ME(gamma, alphas, t(AX[4,]));


		## x = (0,0);

			fmeff_x00.obj   <- meffs_x00;
			fmeff_x00.dir   <- f_x.dir;
			fmeff_x00.rhs  	<- t(cbind(t(pAY_x00),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x00.con	<- rbind(piAY_x00, t(rep(1, M)), diag(1, M));

			LPlmeff_x00     <- lp("min", fmeff_x00.obj, fmeff_x00.con, fmeff_x00.dir, fmeff_x00.rhs);
			lprhos_x00      <- matrix(LPlmeff_x00$solution, ncol = 1);

			if (LPlmeff_x00$status == 0) {; 
				lmeffs_x00_npmle[i,j]	<- LPlmeff_x00$objval;
				};
				
			if (LPlmeff_x00$status == 2) {;
				lmeffs_x00_npmle[i,j]	<- NaN;
				lnosol_x00             	<- lnosol_x00 + 1;
				};

			LPumeff_x00   <- lp("max", fmeff_x00.obj, fmeff_x00.con, fmeff_x00.dir, fmeff_x00.rhs);
			uprhos_x00    <- matrix(LPumeff_x00$solution, ncol = 1);
			if (LPumeff_x00$status == 0) {; 
				umeffs_x00_npmle[i,j]	<- LPumeff_x00$objval;
				};
				
			if (LPumeff_x00$status == 2) {;
				umeffs_x00_npmle[i,j]	<- NaN;
				unosol_x00             	<- unosol_x00 + 1;
				};


		## x = (0,1);

			lmeffs_x01_npmle[i,j]	<- pAY_x01[2,1] - pAY_x01[3,1];
			umeffs_x01_npmle[i,j]	<- pAY_x01[2,1] - pAY_x01[3,1];

		## x = (1,0);

			lmeffs_x10_npmle[i,j]	<- pAY_x10[3,1] - pAY_x10[2,1];
			umeffs_x10_npmle[i,j]	<- pAY_x10[3,1] - pAY_x10[2,1];


		## x = (1,1);

			fmeff_x11.obj   <- meffs_x11;
			fmeff_x11.dir   <- f_x.dir;
			fmeff_x11.rhs  	<- t(cbind(t(pAY_x11),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x11.con	<- rbind(piAY_x11, t(rep(1, M)), diag(1, M));

			LPlmeff_x11     <- lp("min", fmeff_x11.obj, fmeff_x11.con, fmeff_x11.dir, fmeff_x11.rhs);
			lprhos_x11      <- matrix(LPlmeff_x11$solution, ncol = 1);

			if (LPlmeff_x11$status == 0) {; 
				lmeffs_x11_npmle[i,j]	<- LPlmeff_x11$objval;
				};
				
			if (LPlmeff_x11$status == 2) {;
				lmeffs_x11_npmle[i,j]	<- NaN;
				lnosol_x11             	<- lnosol_x11 + 1;
				};

			LPumeff_x11   <- lp("max", fmeff_x11.obj, fmeff_x11.con, fmeff_x11.dir, fmeff_x11.rhs);
			uprhos_x11    <- matrix(LPumeff_x11$solution, ncol = 1);
			if (LPumeff_x11$status == 0) {; 
				umeffs_x11_npmle[i,j]	<- LPumeff_x11$objval;
				};
				
			if (LPumeff_x11$status == 2) {;
				umeffs_x11_npmle[i,j]	<- NaN;
				unosol_x11             	<- unosol_x11 + 1;
				};




	umeff_x00_npmle  <- apply(umeffs_x00_npmle,1,max);
	umeff_x01_npmle  <- apply(umeffs_x01_npmle,1,max);
	umeff_x10_npmle  <- apply(umeffs_x10_npmle,1,max);
	umeff_x11_npmle  <- apply(umeffs_x11_npmle,1,max);
	umeff_npmle	 <- pAX[1,1]*umeff_x00_npmle + pAX[2,1]*umeff_x01_npmle + pAX[3,1]*umeff_x10_npmle + pAX[4,1]*umeff_x11_npmle;

	lmeff_x00_npmle  <- apply(lmeffs_x00_npmle,1,min);
	lmeff_x01_npmle  <- apply(lmeffs_x01_npmle,1,min);
	lmeff_x10_npmle  <- apply(lmeffs_x10_npmle,1,min);
	lmeff_x11_npmle  <- apply(lmeffs_x11_npmle,1,min);
	lmeff_npmle	 <- pAX[1,1]*lmeff_x00_npmle + pAX[2,1]*lmeff_x01_npmle + pAX[3,1]*lmeff_x10_npmle + pAX[4,1]*lmeff_x11_npmle;


	umeff_x00.b  	<- c(umeff_x00.b, umeff_x00_npmle);
	umeff_x01.b  	<- c(umeff_x01.b, umeff_x01_npmle);
	umeff_x10.b  	<- c(umeff_x10.b, umeff_x10_npmle);
	umeff_x11.b  	<- c(umeff_x11.b, umeff_x11_npmle);
	umeff.b		<- c(umeff.b, umeff_npmle);

	lmeff_x00.b  	<- c(lmeff_x00.b, lmeff_x00_npmle);
	lmeff_x01.b  	<- c(lmeff_x01.b, lmeff_x01_npmle);
	lmeff_x10.b  	<- c(lmeff_x10.b, lmeff_x10_npmle);
	lmeff_x11.b  	<- c(lmeff_x11.b, lmeff_x11_npmle);
	lmeff.b		<- c(lmeff.b, lmeff_npmle);

	gamma.b		<- c(gamma.b, gamma0);
	ugamma.b	<- c(ugamma.b, max(gamma));
	lgamma.b	<- c(lgamma.b, min(gamma));

	};


gamma.m		<- c(gamma.m, mean(gamma.b));
lgamma.m	<- c(lgamma.m, quantile(lgamma.b, alpha/2 - beta/2, na.rm = TRUE));
ugamma.m	<- c(ugamma.m, quantile(ugamma.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x00.m	<- c(lmeff_x00.m, quantile(lmeff_x00.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x00.m	<- c(umeff_x00.m, quantile(umeff_x00.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x01.m	<- c(lmeff_x01.m, quantile(lmeff_x01.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x01.m	<- c(umeff_x01.m, quantile(umeff_x01.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x10.m	<- c(lmeff_x10.m, quantile(lmeff_x10.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x10.m	<- c(umeff_x10.m, quantile(umeff_x10.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x11.m	<- c(lmeff_x11.m, quantile(lmeff_x11.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x11.m	<- c(umeff_x11.m, quantile(umeff_x11.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff.m		<- c(lmeff.m, quantile(lmeff.b, alpha/2 - beta/2, na.rm = TRUE));
umeff.m		<- c(umeff.m, quantile(umeff.b, 1 - alpha/2 + beta/2, na.rm = TRUE));


};


mean(gamma.m);
min(lgamma.m);
max(ugamma.m);
min(lmeff_x00.m);
max(umeff_x00.m);
min(lmeff_x01.m);
max(umeff_x01.m);
min(lmeff_x10.m);
max(umeff_x10.m);
min(lmeff_x11.m);
max(umeff_x11.m);
min(lmeff.m);
max(umeff.m);

# 95% Confidence interval;

# Parameter;
print(min(lgamma.m));
print(max(ugamma.m));

# AME;
print(min(lmeff.m));
print(max(umeff.m));

detach(data);


############################;
### MODIFIED PROJECTION;
############################;



library(stats);
library(lpSolve);
library(quadprog);
library(foreign);

# Reading dataset;
data	<- read.dta("data_static_T2.dta");
n	<- nrow(data);
attach(data);

# Parameters of the problem

TT      <- 2;
alpha0s <- c(-Inf,4*(-10:10)/10,Inf);
alphas 	<- c(-Inf,(-80:80)/10,Inf);
M      	<- length(alphas);
M0	<- length(alpha0s);

A      <- t(cbind(c(0,0), c(0,1), c(1,0), c(1,1)));	
AY     <- A;
AX     <- A;

pAX	    <- matrix(0, nrow = 2^TT, ncol = 1);
pAX[1,1]    <- mean((x==1));
pAX[2,1]    <- mean((x==2));
pAX[3,1]    <- mean((x==3));
pAX[4,1]    <- mean((x==4));

nAY	<- 2^TT;
nAX	<- 2^TT;


gridgamma0 <- 1;

set.seed(888);   

nB	 	<- 5000;
alpha		<- .05;

gamma.b		<- NULL;
lgamma.b	<- NULL;
ugamma.b	<- NULL;

umeff_x00.b	<- NULL;
lmeff_x00.b	<- NULL;
umeff_x01.b	<- NULL;
lmeff_x01.b	<- NULL;
umeff_x10.b	<- NULL;
lmeff_x10.b	<- NULL;
umeff_x11.b	<- NULL;
lmeff_x11.b	<- NULL;

lmeff.b		<- NULL;
umeff.b		<- NULL;


gammas	  <- c(-200:10)/200;
gridgamma <- length(gammas);
f_x.dir <- t(cbind(t(rep("=",nAY+1)),t(rep(">=", M))));


pAX0		<- pAX;

reject		<- 0;

for (b in 1:nB) {;

chisqtest	<- 0;
	
while (chisqtest == 0) {;

	pAX1		<- pAX0;	# Fixes probabilities of X to the observed probabilities in the dataset
	nnAX1		<- n*pAX;
	pAY_x00		<- rmultinom(1, size = nnAX1[1,1], prob = PAY(x,y,1,TT))/nnAX1[1,1];
	pAY_x01		<- rmultinom(1, size = nnAX1[2,1], prob = PAY(x,y,2,TT))/nnAX1[2,1];
	pAY_x10		<- rmultinom(1, size = nnAX1[3,1], prob = PAY(x,y,3,TT))/nnAX1[3,1];
	pAY_x11		<- rmultinom(1, size = nnAX1[4,1], prob = PAY(x,y,4,TT))/nnAX1[4,1];
	chisqstat	<- n * ( sum( (pAX1[1,1] * pAY_x00 - pAX0[1,1] * PAY(x,y,1,TT))^2 / (pAX0[1,1] * PAY(x,y,1,TT))) +  sum( (pAX1[2,1] * pAY_x01 - pAX0[2,1] * PAY(x,y,2,TT))^2 / (pAX0[2,1] * PAY(x,y,2,TT))) +  sum( (pAX1[3,1] * pAY_x10 - pAX0[3,1] * PAY(x,y,3,TT))^2 / (pAX0[3,1] * PAY(x,y,3,TT))) +  sum( (pAX1[4,1] * pAY_x11 - pAX0[4,1] * PAY(x,y,4,TT))^2 / (pAX0[4,1] * PAY(x,y,4,TT))) );
	if ( (chisqstat < qchisq(1-alpha, nAX*(nAY-1))) & (chisqstat > qchisq(1-2*alpha, nAX*(nAY-1))) ) chisqtest <- 1;
	if ( chisqstat >= qchisq(1-alpha, nAX*(nAY-1)) | chisqstat <= qchisq(1-2*alpha, nAX*(nAY-1)) ) reject <- reject + 1;
	};

	OBJ    <- matrix(999, gridgamma0, gridgamma);

	umeffs_x00_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x01_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x10_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x11_npmle <- matrix(-999, gridgamma0, gridgamma);

	lmeffs_x00_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x01_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x10_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x11_npmle <- matrix(999, gridgamma0, gridgamma);

	i	<- 1;


###############################;
## RIDGE QUADRATIC PROGRAMMING;
###############################;

MM   <- M0;
lambda	<- 1/(n*log(n));

alpha0s_x00	<- alpha0s;
alpha0s_x01	<- alpha0s;
alpha0s_x10	<- alpha0s;
alpha0s_x11	<- alpha0s;

for (j in 1:gridgamma) {;

	gamma <- gammas[j];


	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma, alpha0s);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma, alpha0s);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma, alpha0s);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma, alpha0s);

	MM_x00	   <- length(alpha0s);
	MM_x01	   <- length(alpha0s);
	MM_x10	   <- length(alpha0s);
	MM_x11	   <- length(alpha0s);

# 1st iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11 ));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00), (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01))), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma, alpha0s_x11, as.vector(prhos_x11));


# 2nd iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]   <- 2*n*QP$value;

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma, alpha0s_x11, as.vector(prhos_x11));


# Final iteration;

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]   <- 2*n*QP$value;


	};

	gamma0	  <- gammas[rank(OBJ) == 1];

	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma0, alpha0s_x00);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma0, alpha0s_x01);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma0, alpha0s_x10);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma0, alpha0s_x11);

	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11 ));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00), (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01))), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));


	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	ppAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	ppAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	ppAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	ppAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));


	aa1_x00    <- sqrt(pAX[1, ]) * cbind((piAY_x00 - kronecker( pAY_x00, matrix(1, nrow = 1, ncol = MM_x00))) / kronecker( sqrt(ppAY_x00), matrix(1, nrow = 1, ncol = MM_x00)), matrix(0, nAY, MM_x01 + MM_x10 + MM_x11));
	aa1_x01    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM_x00),   (piAY_x01 - kronecker(pAY_x01, matrix(1, nrow = 1, ncol = MM_x01)))  / kronecker( sqrt(ppAY_x01), matrix(1, nrow = 1, ncol = MM_x01)), matrix(0, nAY, MM_x10 + MM_x11));
	aa1_x10    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01), (piAY_x10 - kronecker( pAY_x10, matrix(1, nrow = 1, ncol = MM_x10))) / kronecker( sqrt(ppAY_x10), matrix(1, nrow = 1, ncol = MM_x10)), matrix(0, nAY, MM_x11));
	aa1_x11    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, MM_x00 + MM_x01 + MM_x10), (piAY_x11 - kronecker( pAY_x11, matrix(1, nrow = 1, ncol = MM_x11))) / kronecker( sqrt(ppAY_x11), matrix(1, nrow = 1, ncol = MM_x11)));

	A   <- rbind(aa1_x00, aa1_x01, aa1_x10, aa1_x11);
	B   <- rbind(0* (pAY_x00 - piAY_x00[,1]), 0 * (pAY_x01 - piAY_x01[,1]), 0 * (pAY_x10 - piAY_x10[,1]),  0 * (pAY_x11 - piAY_x11[,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*%  B;
	Amat   <- t(rbind(cbind(matrix(-1, nrow=1, ncol = MM_x00), matrix(0, nrow=1, ncol = MM_x01 + MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00), matrix(-1, nrow=1, ncol = MM_x01), matrix(0, nrow=1, ncol = MM_x10 + MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01), matrix(-1, nrow=1, ncol = MM_x10), matrix(0, nrow=1, ncol = MM_x11)), cbind(matrix(0, nrow=1, ncol = MM_x00 + MM_x01 + MM_x10), matrix(-1, nrow=1, ncol = MM_x11)), diag(1, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = MM_x00 + MM_x01 + MM_x10 + MM_x11, ncol = 1));
	meq    <- nAX;
 
	QP 	   <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	solution    <- (QP$solution > 0) * QP$solution;	
	prhos_x00   <- matrix(solution[1:MM_x00], nrow = MM_x00, ncol = 1, byrow = FALSE);
	prhos_x01   <- matrix(solution[(1+MM_x00):(MM_x00+MM_x01)], nrow = MM_x01, ncol = 1, byrow = FALSE);
	prhos_x10   <- matrix(solution[(1+MM_x00+MM_x01):(MM_x00+MM_x01+MM_x10)], nrow = MM_x10, ncol = 1, byrow = FALSE);
	prhos_x11   <- matrix(solution[(1+MM_x00+MM_x01+MM_x10):(MM_x00+MM_x01+MM_x10+MM_x11)], nrow = MM_x11, ncol = 1, byrow = FALSE);

	pAY_x00     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s_x00, as.vector(prhos_x00));
	pAY_x01     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s_x01, as.vector(prhos_x01));
	pAY_x10     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s_x10, as.vector(prhos_x10));
	pAY_x11     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s_x11, as.vector(prhos_x11));


	piAY_x00  <- PIAY(AY, t(AX[1,]), gamma0, alphas);
	piAY_x01  <- PIAY(AY, t(AX[2,]), gamma0, alphas);
	piAY_x10  <- PIAY(AY, t(AX[3,]), gamma0, alphas);
	piAY_x11  <- PIAY(AY, t(AX[4,]), gamma0, alphas);

	gamma     <- gamma0;


	## INDIVIDUAL MARGINAL EFFECTS

	# Using the same notation as for model parameters
		# x = (z_{1}, ..., z_{M})   M unknows
	# Constraints and objective function: 
		# Min/Max_{z} sum_{j}  sum_{m} z_{m} me(a_{m}|gamma,x^{j} | X_{1}, ..., X_{T})
		# s.t  sum_{m} z_{m} pi(A_{i} | x^{j}, \gamma) = P(A_{i}| x^{j}, \gamma_{0})    nAY constraints
		#      sum_{m} z_{m} = 1                                                        1 constraints
		#      z_{m} >= 0								M constraints


	J	<- 1;

	meffs_x00    <- ME(gamma, alphas, t(AX[1,]));
	meffs_x01    <- ME(gamma, alphas, t(AX[2,]));
	meffs_x10    <- ME(gamma, alphas, t(AX[3,]));
	meffs_x11    <- ME(gamma, alphas, t(AX[4,]));


		## x = (0,0);

			fmeff_x00.obj   <- meffs_x00;
			fmeff_x00.dir   <- f_x.dir;
			fmeff_x00.rhs  	<- t(cbind(t(pAY_x00),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x00.con	<- rbind(piAY_x00, t(rep(1, M)), diag(1, M));

			LPlmeff_x00     <- lp("min", fmeff_x00.obj, fmeff_x00.con, fmeff_x00.dir, fmeff_x00.rhs);
			lprhos_x00      <- matrix(LPlmeff_x00$solution, ncol = 1);

			if (LPlmeff_x00$status == 0) {; 
				lmeffs_x00_npmle[i,j]	<- LPlmeff_x00$objval;
				};
				
			if (LPlmeff_x00$status == 2) {;
				lmeffs_x00_npmle[i,j]	<- NaN;
				lnosol_x00             	<- lnosol_x00 + 1;
				};

			LPumeff_x00   <- lp("max", fmeff_x00.obj, fmeff_x00.con, fmeff_x00.dir, fmeff_x00.rhs);
			uprhos_x00    <- matrix(LPumeff_x00$solution, ncol = 1);
			if (LPumeff_x00$status == 0) {; 
				umeffs_x00_npmle[i,j]	<- LPumeff_x00$objval;
				};
				
			if (LPumeff_x00$status == 2) {;
				umeffs_x00_npmle[i,j]	<- NaN;
				unosol_x00             	<- unosol_x00 + 1;
				};


		## x = (0,1);

			lmeffs_x01_npmle[i,j]	<- pAY_x01[2,1] - pAY_x01[3,1];
			umeffs_x01_npmle[i,j]	<- pAY_x01[2,1] - pAY_x01[3,1];

		## x = (1,0);

			lmeffs_x10_npmle[i,j]	<- pAY_x10[3,1] - pAY_x10[2,1];
			umeffs_x10_npmle[i,j]	<- pAY_x10[3,1] - pAY_x10[2,1];


		## x = (1,1);

			fmeff_x11.obj   <- meffs_x11;
			fmeff_x11.dir   <- f_x.dir;
			fmeff_x11.rhs  	<- t(cbind(t(pAY_x11),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x11.con	<- rbind(piAY_x11, t(rep(1, M)), diag(1, M));

			LPlmeff_x11     <- lp("min", fmeff_x11.obj, fmeff_x11.con, fmeff_x11.dir, fmeff_x11.rhs);
			lprhos_x11      <- matrix(LPlmeff_x11$solution, ncol = 1);

			if (LPlmeff_x11$status == 0) {; 
				lmeffs_x11_npmle[i,j]	<- LPlmeff_x11$objval;
				};
				
			if (LPlmeff_x11$status == 2) {;
				lmeffs_x11_npmle[i,j]	<- NaN;
				lnosol_x11             	<- lnosol_x11 + 1;
				};

			LPumeff_x11   <- lp("max", fmeff_x11.obj, fmeff_x11.con, fmeff_x11.dir, fmeff_x11.rhs);
			uprhos_x11    <- matrix(LPumeff_x11$solution, ncol = 1);
			if (LPumeff_x11$status == 0) {; 
				umeffs_x11_npmle[i,j]	<- LPumeff_x11$objval;
				};
				
			if (LPumeff_x11$status == 2) {;
				umeffs_x11_npmle[i,j]	<- NaN;
				unosol_x11             	<- unosol_x11 + 1;
				};




	umeff_x00_npmle  <- apply(umeffs_x00_npmle,1,max);
	umeff_x01_npmle  <- apply(umeffs_x01_npmle,1,max);
	umeff_x10_npmle  <- apply(umeffs_x10_npmle,1,max);
	umeff_x11_npmle  <- apply(umeffs_x11_npmle,1,max);
	umeff_npmle	 <- pAX[1,1]*umeff_x00_npmle + pAX[2,1]*umeff_x01_npmle + pAX[3,1]*umeff_x10_npmle + pAX[4,1]*umeff_x11_npmle;

	lmeff_x00_npmle  <- apply(lmeffs_x00_npmle,1,min);
	lmeff_x01_npmle  <- apply(lmeffs_x01_npmle,1,min);
	lmeff_x10_npmle  <- apply(lmeffs_x10_npmle,1,min);
	lmeff_x11_npmle  <- apply(lmeffs_x11_npmle,1,min);
	lmeff_npmle	 <- pAX[1,1]*lmeff_x00_npmle + pAX[2,1]*lmeff_x01_npmle + pAX[3,1]*lmeff_x10_npmle + pAX[4,1]*lmeff_x11_npmle;


	umeff_x00.b  	<- c(umeff_x00.b, umeff_x00_npmle);
	umeff_x01.b  	<- c(umeff_x01.b, umeff_x01_npmle);
	umeff_x10.b  	<- c(umeff_x10.b, umeff_x10_npmle);
	umeff_x11.b  	<- c(umeff_x11.b, umeff_x11_npmle);
	umeff.b		<- c(umeff.b, umeff_npmle);

	lmeff_x00.b  	<- c(lmeff_x00.b, lmeff_x00_npmle);
	lmeff_x01.b  	<- c(lmeff_x01.b, lmeff_x01_npmle);
	lmeff_x10.b  	<- c(lmeff_x10.b, lmeff_x10_npmle);
	lmeff_x11.b  	<- c(lmeff_x11.b, lmeff_x11_npmle);
	lmeff.b		<- c(lmeff.b, lmeff_npmle);



	gamma.b		<- c(gamma.b, gamma0);
	ugamma.b	<- c(ugamma.b, max(gamma));
	lgamma.b	<- c(lgamma.b, min(gamma));

	};


min(gamma.b);
max(gamma.b);
min(lmeff_x00.b);
max(umeff_x00.b);
min(lmeff_x01.b);
max(umeff_x01.b);
min(lmeff_x10.b);
max(umeff_x10.b);
min(lmeff_x11.b);
max(umeff_x11.b);
min(lmeff.b);
max(umeff.b);

# 95% Confidence interval;

# Parameter;
print(min(gamma.b));
print(max(gamma.b));

# AME;
print(min(lmeff.b));
print(max(umeff.b));

detach(data);

######################################################################################################################################
######                                                      T = 3
######################################################################################################################################

#############################################;
# POINT ESTIMATES AND CANONICAL PROJECTION;
#############################################;


library(stats);
library(lpSolve);
library(quadprog);
library(foreign);

# Reading dataset;
data  <- read.dta("data_static_T3.dta");
n	<- nrow(data);
attach(data);

# Parameters of the problem

TT	<- 3;
alpha0s <- c(-Inf,4*(-10:10)/10,Inf);
alphas 	<- c(-Inf,(-80:80)/10,Inf);
M0	<- length(alpha0s);
M      	<- length(alphas);

A      <- t(cbind(c(0,0,0), c(0,0,1), c(0,1,0), c(1,0,0), c(0,1,1), c(1,0,1), c(1,1,0), c(1,1,1)));
AY     <- A;
AX     <- A;

pAX	    <- matrix(0, nrow = 2^TT, ncol = 1);
pAX[1,1]    <- mean((x==1));
pAX[2,1]    <- mean((x==2));
pAX[3,1]    <- mean((x==3));
pAX[4,1]    <- mean((x==4));
pAX[5,1]    <- mean((x==5));
pAX[6,1]    <- mean((x==6));
pAX[7,1]    <- mean((x==7));
pAX[8,1]    <- mean((x==8));


# Parameters of the optimization

gridgamma0 <- 1;
gridgamma <- 301;
ugamma    <- gridgamma - 1;
gammas    <- 1.5*(-ugamma:0)/(gridgamma-1);

# Initialization of variables

OBJ    <- matrix(-999, gridgamma0, gridgamma);
OBJ2  <- matrix(-999, gridgamma0, gridgamma);

meffs_npmle    <- matrix(-999, gridgamma0, gridgamma);

umeffs_x000_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x001_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x010_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x100_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x011_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x101_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x110_npmle <- matrix(-999, gridgamma0, gridgamma);
umeffs_x111_npmle <- matrix(-999, gridgamma0, gridgamma);

lmeffs_x000_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x001_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x010_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x100_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x011_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x101_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x110_npmle <- matrix(999, gridgamma0, gridgamma);
lmeffs_x111_npmle <- matrix(999, gridgamma0, gridgamma);


extreme_npmle         <- matrix(-999, gridgamma0, gridgamma);
lextreme_npmle        <- matrix(-999, gridgamma0, gridgamma);
uextreme_npmle        <- matrix(-999, gridgamma0, gridgamma);
support_alpha_npmle   <- matrix(-999, gridgamma0, gridgamma);
usupport_alpha_x000_npmle  <- matrix(-999, gridgamma0, gridgamma);
lsupport_alpha_x000_npmle  <- matrix(-999, gridgamma0, gridgamma);
usupport_alpha_x111_npmle  <- matrix(-999, gridgamma0, gridgamma);
lsupport_alpha_x111_npmle  <- matrix(-999, gridgamma0, gridgamma);


nAY  <- nrow(AY);
nAX  <- nrow(AX);
T    <- ncol(AX);

f.obj <- t(cbind(t(rep(1,nAY*nAX+nAX)),t(rep(0,nAX*M))));
f.dir <- t(cbind(t(rep("=",nAY*nAX+nAX)),t(rep(">=", nAX*(nAY+1+M)))));
f_x.dir <- t(cbind(t(rep("=",nAY+1)),t(rep(">=", M))));


lnosol <- 0;
unosol <- 0;

lnosol_x000 <- 0;
unosol_x000 <- 0;
lnosol_x111 <- 0;
unosol_x111 <- 0;


i <- 1;

# (1) ORIGINAL DATA


	pAY_x000     <- PAY(x, y, 1, TT);
	pAY_x001     <- PAY(x, y, 2, TT);
	pAY_x010     <- PAY(x, y, 3, TT);
	pAY_x100     <- PAY(x, y, 4, TT);
	pAY_x011     <- PAY(x, y, 5, TT);
	pAY_x101     <- PAY(x, y, 6, TT);
	pAY_x110     <- PAY(x, y, 7, TT);
	pAY_x111     <- PAY(x, y, 8, TT);


###############################;
## RIDGE QUADRATIC PROGRAMMING;
###############################;

lambda	<- 1/(n*log(n));

MM	<- length(alpha0s);


for (j in 1:gridgamma) {;

	gamma <- gammas[j];

	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma, alpha0s);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma, alpha0s);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma, alpha0s);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma, alpha0s);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma, alpha0s);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma, alpha0s);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma, alpha0s);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma, alpha0s);

# 1st iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))));

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;

	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma, alpha0s, as.vector(prhos_x[, 8]));

# 2nd iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma, alpha0s, as.vector(prhos_x[, 8]));

# Final iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;

	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma, alpha0s, as.vector(prhos_x[, 8]));

	OBJ2[i,j]  <- n * (t( rbind( (pAY_x000-ppAY_x000) / sqrt(ppAY_x000), (pAY_x001-ppAY_x001) / sqrt(ppAY_x001), (pAY_x010-ppAY_x010) / sqrt(ppAY_x010), (pAY_x100-ppAY_x100) / sqrt(ppAY_x100), (pAY_x011-ppAY_x011) / sqrt(ppAY_x011), (pAY_x101-ppAY_x101) / sqrt(ppAY_x101), (pAY_x110-ppAY_x110) / sqrt(ppAY_x110), (pAY_x111-ppAY_x111) / sqrt(ppAY_x111) ) ) 
			%*% kronecker(diag(as.vector(pAX)), diag(1, nrow = nAX)) 
			%*% rbind( (pAY_x000-ppAY_x000) / sqrt(ppAY_x000), (pAY_x001-ppAY_x001) / sqrt(ppAY_x001), (pAY_x010-ppAY_x010) / sqrt(ppAY_x010), (pAY_x100-ppAY_x100) / sqrt(ppAY_x100), (pAY_x011-ppAY_x011) / sqrt(ppAY_x011), (pAY_x101-ppAY_x101) / sqrt(ppAY_x101), (pAY_x110-ppAY_x110) / sqrt(ppAY_x110), (pAY_x111-ppAY_x111) / sqrt(ppAY_x111) ) );
	};

	gamma0	  <- gammas[rank(OBJ) == 1];
  gammas0.cp	  <- gammas[OBJ2 <= qchisq(.95, nAX*(nAY-1))];

	pAY_x000     <- PAY(x, y, 1, TT);
	pAY_x001     <- PAY(x, y, 2, TT);
	pAY_x010     <- PAY(x, y, 3, TT);
	pAY_x100     <- PAY(x, y, 4, TT);
	pAY_x011     <- PAY(x, y, 5, TT);
	pAY_x101     <- PAY(x, y, 6, TT);
	pAY_x110     <- PAY(x, y, 7, TT);
	pAY_x111     <- PAY(x, y, 8, TT);


	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma0, alpha0s);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma0, alpha0s);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma0, alpha0s);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma0, alpha0s);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma0, alpha0s);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma0, alpha0s);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma0, alpha0s);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma0, alpha0s);

# 1st iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))));

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));

# 2nd iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));

# Final iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	pAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	pAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	pAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	pAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	pAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	pAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	pAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	pAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));


# LINEAR PROGRAMMING;


	# Optimization LP problem
		# Solves max (f.obj)'x
		#         st (f.con) x "f.dir" f.rhs
        	# where x is the vector of unknowns
        	#       f.obj is the vector of coefficients in the objective function
        	#       f.con is the matrix of coefficients of the linear constraints
        	#       f.dir is the vector of directions of the constraints
        	#       f.rhs is the vector of coefficients in the rhs of the constraints

	## MODEL PARAMETERS
	# In this example for the model parameters the vector of unknowns is sorted (in the notation of the paper)
		# x = (w_{1}, ..., w_{nAX}, v_{1,1}, ..., v_{nAY,nAX}, z_{1}, ..., z_{nAX*M})   nAX *(1 + nAY + M) unknows
		# Constraints and objective function: 
		# Min_{w,v,z} sum_{j} w_{j} + sum_{i,j} v_{i,j}
		# s.t  v_{i,j} + sum_{m} z_{(j-1)*M + m} pi(A_{i} | x^{j}, \gamma) = P(A_{i}| x^{j}, \gamma_{0})    nAY*nAX constraints
		#      w_{j} + sum_{m} z_{(j-1)*M + m} = 1                                                          nAX constraints
		#      v_{i,j}, w_{j}, z_{m} >= 0                                                                   nAX *(1 + nAY + M) constraints


	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma0, alphas);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma0, alphas);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma0, alphas);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma0, alphas);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma0, alphas);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma0, alphas);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma0, alphas);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma0, alphas);

	a1_x000    <- cbind(matrix(0, nAY, nAX), diag(rep(1,nAY)), matrix(0, nAY, 7*nAY), piAY_x000, matrix(0, nAY, 7*M));
	a1_x001    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, nAY), diag(rep(1,nAY)), matrix(0, nAY, 6*nAY), matrix(0, nAY, M), piAY_x001, matrix(0, nAY, 6*M));
	a1_x010    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 2*nAY), diag(rep(1,nAY)), matrix(0, nAY, 5*nAY), matrix(0, nAY, 2*M), piAY_x010, matrix(0, nAY, 5*M));
	a1_x100    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 3*nAY), diag(rep(1,nAY)), matrix(0, nAY, 4*nAY), matrix(0, nAY, 3*M), piAY_x100, matrix(0, nAY, 4*M));
	a1_x011    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 4*nAY), diag(rep(1,nAY)), matrix(0, nAY, 3*nAY), matrix(0, nAY, 4*M), piAY_x011, matrix(0, nAY, 3*M));
	a1_x101    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 5*nAY), diag(rep(1,nAY)), matrix(0, nAY, 2*nAY), matrix(0, nAY, 5*M), piAY_x101, matrix(0, nAY, 2*M));
	a1_x110    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 6*nAY), diag(rep(1,nAY)), matrix(0, nAY, nAY), matrix(0, nAY, 6*M), piAY_x110, matrix(0, nAY, M));
	a1_x111    <- cbind(matrix(0, nAY, nAX), matrix(0, nAY, 7*nAY), diag(rep(1,nAY)), matrix(0, nAY, 7*M), piAY_x111);
	a2_x000    <- cbind(matrix(c(1,0,0,0,0,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(1, M)),  t(rep(0, 7*M)));
	a2_x001    <- cbind(matrix(c(0,1,0,0,0,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, M)), t(rep(1, M)),  t(rep(0, 6*M)));
	a2_x010    <- cbind(matrix(c(0,0,1,0,0,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 2*M)), t(rep(1, M)),  t(rep(0, 5*M)));
	a2_x100    <- cbind(matrix(c(0,0,0,1,0,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 3*M)), t(rep(1, M)),  t(rep(0, 4*M)));
	a2_x011    <- cbind(matrix(c(0,0,0,0,1,0,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 4*M)), t(rep(1, M)),  t(rep(0, 3*M)));
	a2_x101    <- cbind(matrix(c(0,0,0,0,0,1,0,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 5*M)), t(rep(1, M)),  t(rep(0, 2*M)));
	a2_x110    <- cbind(matrix(c(0,0,0,0,0,0,1,0),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 6*M)), t(rep(1, M)),  t(rep(0, M)));
	a2_x111    <- cbind(matrix(c(0,0,0,0,0,0,0,1),1,nAX), t(rep(0, nAX*nAY)), t(rep(0, 7*M)), t(rep(1, M)));
        a3         <- diag(rep(1, nAX*(1 + nAY + M)));
        f.con      <- rbind(a1_x000, a1_x001,a1_x010,a1_x100,a1_x011,a1_x101,a1_x110,a1_x111,a2_x000, a2_x001, a2_x010, a2_x100, a2_x011, a2_x101, a2_x110, a2_x111, a3);
	f.rhs        <- t(cbind(t(pAY_x000), t(pAY_x001),t(pAY_x010),t(pAY_x100),t(pAY_x011),t(pAY_x101),t(pAY_x110),t(pAY_x111),t(rep(1, nAX)), t(rep(0, nAX*(1+nAY+M)))));

	LP        <- lp("min", f.obj, f.con, f.dir, f.rhs);


	gamma	<- gamma0;

	## INDIVIDUAL MARGINAL EFFECTS
	# Using the same notation as for model parameters
		# x = (z_{1}, ..., z_{M})   M unknows
	# Constraints and objective function: 
		# Min/Max_{z} sum_{j}  sum_{m} z_{m} me(a_{m}|gamma,x^{j})
		# s.t  sum_{m} z_{m} pi(A_{i} | x^{j}, \gamma) = P(A_{i}| x^{j}, \gamma_{0})    nAY constraints
		#      sum_{m} z_{m} = 1                                                        1 constraints
		#      z_{m} >= 0								M constraints

		meffs_x000    <- ME(gamma, alphas, t(AX[1,]));
		meffs_x001    <- ME(gamma, alphas, t(AX[2,]));
		meffs_x010    <- ME(gamma, alphas, t(AX[3,]));
		meffs_x100    <- ME(gamma, alphas, t(AX[4,]));
		meffs_x011    <- ME(gamma, alphas, t(AX[5,]));
		meffs_x101    <- ME(gamma, alphas, t(AX[6,]));
		meffs_x110    <- ME(gamma, alphas, t(AX[7,]));
		meffs_x111    <- ME(gamma, alphas, t(AX[nrow(AX),]));


		## x = (0,0,0);

			fmeff_x000.obj  <- meffs_x000;
			fmeff_x000.dir  <- f_x.dir;
			fmeff_x000.rhs  <- t(cbind(t(pAY_x000),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x000.con	<- rbind(piAY_x000, t(rep(1, M)), diag(1, M));

			LPlmeff_x000     <- lp("min", fmeff_x000.obj, fmeff_x000.con, fmeff_x000.dir, fmeff_x000.rhs);
			lprhos_x000      <- matrix(LPlmeff_x000$solution, ncol = 1);

			if (LPlmeff_x000$status == 0) {; 
				lmeffs_x000_npmle[i,j]	<- LPlmeff_x000$objval;
				};
				
			if (LPlmeff_x000$status == 2) {;
				lmeffs_x000_npmle[i,j]	<- NaN;
				lnosol_x000             	<- lnosol_x000 + 1;
				};

			LPumeff_x000   <- lp("max", fmeff_x000.obj, fmeff_x000.con, fmeff_x000.dir, fmeff_x000.rhs);
			uprhos_x000    <- matrix(LPumeff_x000$solution, ncol = 1);
			if (LPumeff_x000$status == 0) {; 
				umeffs_x000_npmle[i,j]	<- LPumeff_x000$objval;
				};
				
			if (LPumeff_x000$status == 2) {;
				umeffs_x000_npmle[i,j]	<- NaN;
				unosol_x000             <- unosol_x000 + 1;
				};
			lsupport_alpha_x000_npmle[i,j]   <- support(lprhos_x000);
			usupport_alpha_x000_npmle[i,j]   <- support(uprhos_x000);


		## x = (1,1,1);

			fmeff_x111.obj   <- meffs_x111;
			fmeff_x111.dir   <- f_x.dir;
			fmeff_x111.rhs  	<- t(cbind(t(pAY_x111),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x111.con	<- rbind(piAY_x111, t(rep(1, M)), diag(1, M));

			LPlmeff_x111     <- lp("min", fmeff_x111.obj, fmeff_x111.con, fmeff_x111.dir, fmeff_x111.rhs);
			lprhos_x111      <- matrix(LPlmeff_x111$solution, ncol = 1);

			if (LPlmeff_x111$status == 0) {; 
				lmeffs_x111_npmle[i,j]	<- LPlmeff_x111$objval;
				};
				
			if (LPlmeff_x111$status == 2) {;
				lmeffs_x111_npmle[i,j]	<- NaN;
				lnosol_x111             	<- lnosol_x111 + 1;
				};

			LPumeff_x111   <- lp("max", fmeff_x111.obj, fmeff_x111.con, fmeff_x111.dir, fmeff_x111.rhs);
			uprhos_x111    <- matrix(LPumeff_x111$solution, ncol = 1);
			if (LPumeff_x111$status == 0) {; 
				umeffs_x111_npmle[i,j]	<- LPumeff_x111$objval;
				};
				
			if (LPumeff_x111$status == 2) {;
				umeffs_x111_npmle[i,j]	<- NaN;
				unosol_x111             <- unosol_x111 + 1;
				};

			lsupport_alpha_x111_npmle[i,j]   <- support(lprhos_x111);
			usupport_alpha_x111_npmle[i,j]   <- support(uprhos_x111);


      # Nonparametrically identified marginal effects;
      
      F1_x001			<- pAY_x001[4,1] + pAY_x001[6,1] + pAY_x001[7,1] + pAY_x001[8,1];
      F2_x001			<- pAY_x001[3,1] + pAY_x001[5,1] + pAY_x001[7,1] + pAY_x001[8,1];
      F3_x001			<- pAY_x001[2,1] + pAY_x001[5,1] + pAY_x001[6,1] + pAY_x001[8,1];
      umeffs_x001_npmle[i,j]	<- F3_x001 - F1_x001;
      lmeffs_x001_npmle[i,j]	<- F3_x001 - F2_x001;
      
      F1_x010			<- pAY_x010[4,1] + pAY_x010[6,1] + pAY_x010[7,1] + pAY_x010[8,1];
      F2_x010			<- pAY_x010[3,1] + pAY_x010[5,1] + pAY_x010[7,1] + pAY_x010[8,1];
      F3_x010			<- pAY_x010[2,1] + pAY_x010[5,1] + pAY_x010[6,1] + pAY_x010[8,1];
      umeffs_x010_npmle[i,j] 	<- F2_x010 - F1_x010;
      lmeffs_x010_npmle[i,j] 	<- F2_x010 - F3_x010;
      
      F1_x100			<- pAY_x100[4,1] + pAY_x100[6,1] + pAY_x100[7,1] + pAY_x100[8,1];
      F2_x100			<- pAY_x100[3,1] + pAY_x100[5,1] + pAY_x100[7,1] + pAY_x100[8,1];
      F3_x100			<- pAY_x100[2,1] + pAY_x100[5,1] + pAY_x100[6,1] + pAY_x100[8,1];
      umeffs_x100_npmle[i,j] 	<- F1_x100 - F2_x100;
      lmeffs_x100_npmle[i,j] 	<- F1_x100 - F3_x100;
      
      F1_x011			<- pAY_x011[4,1] + pAY_x011[6,1] + pAY_x011[7,1] + pAY_x011[8,1];
      F2_x011			<- pAY_x011[3,1] + pAY_x011[5,1] + pAY_x011[7,1] + pAY_x011[8,1];
      F3_x011			<- pAY_x011[2,1] + pAY_x011[5,1] + pAY_x011[6,1] + pAY_x011[8,1];
      umeffs_x011_npmle[i,j] 	<- F2_x011 - F1_x011;
      lmeffs_x011_npmle[i,j] 	<- F3_x011 - F1_x011;
      
      F1_x101			<- pAY_x101[4,1] + pAY_x101[6,1] + pAY_x101[7,1] + pAY_x101[8,1];
      F2_x101			<- pAY_x101[3,1] + pAY_x101[5,1] + pAY_x101[7,1] + pAY_x101[8,1];
      F3_x101			<- pAY_x101[2,1] + pAY_x101[5,1] + pAY_x101[6,1] + pAY_x101[8,1];
      umeffs_x101_npmle[i,j] 	<- F1_x101 - F2_x101;
      lmeffs_x101_npmle[i,j] 	<- F3_x101 - F2_x101;
      
      F1_x110			<- pAY_x110[4,1] + pAY_x110[6,1] + pAY_x110[7,1] + pAY_x110[8,1];
      F2_x110			<- pAY_x110[3,1] + pAY_x110[5,1] + pAY_x110[7,1] + pAY_x110[8,1];
      F3_x110			<- pAY_x110[2,1] + pAY_x110[5,1] + pAY_x110[6,1] + pAY_x110[8,1];
      umeffs_x110_npmle[i,j] 	<- F1_x110 - F3_x110;
      lmeffs_x110_npmle[i,j] 	<- F2_x110 - F3_x110;


	umeff_x000_npmle  <- apply(umeffs_x000_npmle,1,max);
	umeff_x001_npmle  <- apply(umeffs_x001_npmle,1,max);
	umeff_x010_npmle  <- apply(umeffs_x010_npmle,1,max);
	umeff_x100_npmle  <- apply(umeffs_x100_npmle,1,max);
	umeff_x011_npmle  <- apply(umeffs_x011_npmle,1,max);
	umeff_x101_npmle  <- apply(umeffs_x101_npmle,1,max);
	umeff_x110_npmle  <- apply(umeffs_x110_npmle,1,max);
	umeff_x111_npmle  <- apply(umeffs_x111_npmle,1,max);
	umeff_npmle	 <- pAX[1,1]*umeff_x000_npmle + pAX[2,1]*umeff_x001_npmle + pAX[3,1]*umeff_x010_npmle + pAX[4,1]*umeff_x100_npmle + pAX[5,1]*umeff_x011_npmle + pAX[6,1]*umeff_x101_npmle + pAX[7,1]*umeff_x110_npmle + pAX[8,1]*umeff_x111_npmle;

	lmeff_x000_npmle  <- apply(lmeffs_x000_npmle,1,min);
	lmeff_x001_npmle  <- apply(lmeffs_x001_npmle,1,min);
	lmeff_x010_npmle  <- apply(lmeffs_x010_npmle,1,min);
	lmeff_x100_npmle  <- apply(lmeffs_x100_npmle,1,min);
	lmeff_x011_npmle  <- apply(lmeffs_x011_npmle,1,min);
	lmeff_x101_npmle  <- apply(lmeffs_x101_npmle,1,min);
	lmeff_x110_npmle  <- apply(lmeffs_x110_npmle,1,min);
	lmeff_x111_npmle  <- apply(lmeffs_x111_npmle,1,min);
	lmeff_npmle	 <- pAX[1,1]*lmeff_x000_npmle + pAX[2,1]*lmeff_x001_npmle + pAX[3,1]*lmeff_x010_npmle + pAX[4,1]*lmeff_x100_npmle + pAX[5,1]*lmeff_x011_npmle + pAX[6,1]*lmeff_x101_npmle + pAX[7,1]*lmeff_x110_npmle + pAX[8,1]*lmeff_x111_npmle;

lmeff_x000_npmle
umeff_x000_npmle
lmeff_x001_npmle
umeff_x001_npmle
lmeff_x010_npmle
umeff_x010_npmle
lmeff_x100_npmle
umeff_x100_npmle
lmeff_x011_npmle
umeff_x011_npmle
lmeff_x101_npmle
umeff_x101_npmle
lmeff_x110_npmle
umeff_x110_npmle
lmeff_x111_npmle
umeff_x111_npmle
lmeff_npmle
umeff_npmle

# Coefficient SP-model;
print(gamma0);

# AME SP-bounds;
print(lmeff_npmle);
print(umeff_npmle);

# Canonical projection: Coefficient;
print(min(gammas0.cp));
print(max(gammas0.cp));


detach(data);



##########################;
### PERTURBED BOOTSTRAP;
##########################;



library(stats);
library(lpSolve);
library(quadprog);
library(foreign);

# Reading dataset;
data  <- read.dta("data_static_T3.dta");
n	<- nrow(data);
attach(data);

# Parameters of the problem

TT      <- 3;
alpha0s <- c(-Inf,4*(-10:10)/10,Inf);
alphas 	<- c(-Inf,(-80:80)/10,Inf);
M      	<- length(alphas);
M0	<- length(alpha0s);

A      <- t(cbind(c(0,0,0), c(0,0,1), c(0,1,0), c(1,0,0), c(0,1,1), c(1,0,1), c(1,1,0), c(1,1,1)));
AY     <- A;
AX     <- A;

pAX	    <- matrix(0, nrow = 2^TT, ncol = 1);
pAX[1,1]    <- mean((x==1));
pAX[2,1]    <- mean((x==2));
pAX[3,1]    <- mean((x==3));
pAX[4,1]    <- mean((x==4));
pAX[5,1]    <- mean((x==5));
pAX[6,1]    <- mean((x==6));
pAX[7,1]    <- mean((x==7));
pAX[8,1]    <- mean((x==8));

nAX	    <- 2^TT;
nAY	    <- 2^TT;

# Parameters of the optimization

gridgamma0 <- 1;

set.seed(8888);   

nM		<- 100;
nB	 	<- 200;
beta		<- .01;
alpha		<- .05;

gamma.m		<- NULL;
lgamma.m	<- NULL;
ugamma.m	<- NULL;

umeff_x000.m	<- NULL;
lmeff_x000.m	<- NULL;
umeff_x001.m	<- NULL;
lmeff_x001.m	<- NULL;
umeff_x010.m	<- NULL;
lmeff_x010.m	<- NULL;
umeff_x100.m	<- NULL;
lmeff_x100.m	<- NULL;
umeff_x011.m	<- NULL;
lmeff_x011.m	<- NULL;
umeff_x101.m	<- NULL;
lmeff_x101.m	<- NULL;
umeff_x110.m	<- NULL;
lmeff_x110.m	<- NULL;
umeff_x111.m	<- NULL;
lmeff_x111.m	<- NULL;

lmeff.m		<- NULL;
umeff.m		<- NULL;

gammas	  <- c(-150:50)/100;
gridgamma <- length(gammas);
f_x.dir <- t(cbind(t(rep("=",nAY+1)),t(rep(">=", M))));


pAX0		<- pAX;
reject		<- 0;

for (m in 1:nM) {;


# 1 - PERTURBATION;


chisqtest	<- 0;

while (chisqtest == 0) {;
	pAX1		<- pAX0;	# Fixes probabilities of X to the observed probabilities in the dataset
	nnAX1		<- n*pAX1;
	pAY1_x000	<- rmultinom(1, size = nnAX1[1,1], prob = PAY(x,y,1,TT))/nnAX1[1,1];
	pAY1_x001	<- rmultinom(1, size = nnAX1[2,1], prob = PAY(x,y,2,TT))/nnAX1[2,1];
	pAY1_x010	<- rmultinom(1, size = nnAX1[3,1], prob = PAY(x,y,3,TT))/nnAX1[3,1];
	pAY1_x100	<- rmultinom(1, size = nnAX1[4,1], prob = PAY(x,y,4,TT))/nnAX1[4,1];
	pAY1_x011	<- rmultinom(1, size = nnAX1[5,1], prob = PAY(x,y,5,TT))/nnAX1[5,1];
	pAY1_x101	<- rmultinom(1, size = nnAX1[6,1], prob = PAY(x,y,6,TT))/nnAX1[6,1];
	pAY1_x110	<- rmultinom(1, size = nnAX1[7,1], prob = PAY(x,y,7,TT))/nnAX1[7,1];
	pAY1_x111	<- rmultinom(1, size = nnAX1[8,1], prob = PAY(x,y,8,TT))/nnAX1[8,1];
	chisqstat	<- n * ( sum( (pAX1[1,1] * pAY1_x000 - pAX0[1,1] * PAY(x,y,1,TT))^2 / (pAX1[1,1] * pAY1_x000)) +  sum( (pAX1[2,1] * pAY1_x001[-2,] - pAX0[2,1] * PAY(x,y,2,TT)[-2,])^2 / (pAX1[2,1] * pAY1_x001[-2,])) +  sum( (pAX1[3,1] * pAY1_x010 - pAX0[3,1] * PAY(x,y,3,TT))^2 / (pAX1[3,1] * pAY1_x010)) +  sum( (pAX1[4,1] * pAY1_x100 - pAX0[4,1] * PAY(x,y,4,TT))^2 / (pAX1[4,1] * pAY1_x100)) +  sum( (pAX1[5,1] * pAY1_x011 - pAX0[5,1] * PAY(x,y,5,TT))^2 / (pAX1[5,1] * pAY1_x011)) +  sum( (pAX1[6,1] * pAY1_x101 - pAX0[6,1] * PAY(x,y,6,TT))^2 / (pAX1[6,1] * pAY1_x101)) +  sum( (pAX1[7,1] * pAY1_x110 - pAX0[7,1] * PAY(x,y,7,TT))^2 / (pAX1[7,1] * pAY1_x110)) +  sum( (pAX1[8,1] * pAY1_x111 - pAX0[8,1] * PAY(x,y,8,TT))^2 / (pAX1[8,1] * pAY1_x111)) );
	if (chisqstat < qchisq(1-beta, nAX*(nAY-1)))  chisqtest <- 1;
	if (chisqstat >= qchisq(1-beta, nAX*(nAY-1))) reject <- reject + 1;
	};


# 2 - PARAMETRIC BOOTSTRAP;


gamma.b		<- NULL;
lgamma.b	<- NULL;
ugamma.b	<- NULL;

umeff_x000.b	<- NULL;
lmeff_x000.b	<- NULL;
umeff_x001.b	<- NULL;
lmeff_x001.b	<- NULL;
umeff_x010.b	<- NULL;
lmeff_x010.b	<- NULL;
umeff_x100.b	<- NULL;
lmeff_x100.b	<- NULL;
umeff_x011.b	<- NULL;
lmeff_x011.b	<- NULL;
umeff_x101.b	<- NULL;
lmeff_x101.b	<- NULL;
umeff_x110.b	<- NULL;
lmeff_x110.b	<- NULL;
umeff_x111.b	<- NULL;
lmeff_x111.b	<- NULL;

lmeff.b		<- NULL;
umeff.b		<- NULL;



for (b in 1:nB) {;
	
	pAX1		<- pAX0;	# Fixes probabilities of X to the observed probabilities in the dataset
	nnAX		<- n*pAX1;
	pAY_x000	<- rmultinom(1, size = nnAX[1,1], prob = pAY1_x000)/nnAX[1,1];
	pAY_x001	<- rmultinom(1, size = nnAX[2,1], prob = pAY1_x001)/nnAX[2,1];
	pAY_x010	<- rmultinom(1, size = nnAX[3,1], prob = pAY1_x010)/nnAX[3,1];
	pAY_x100	<- rmultinom(1, size = nnAX[4,1], prob = pAY1_x100)/nnAX[4,1];
	pAY_x011	<- rmultinom(1, size = nnAX[5,1], prob = pAY1_x011)/nnAX[5,1];
	pAY_x101	<- rmultinom(1, size = nnAX[6,1], prob = pAY1_x101)/nnAX[6,1];
	pAY_x110	<- rmultinom(1, size = nnAX[7,1], prob = pAY1_x110)/nnAX[7,1];
	pAY_x111	<- rmultinom(1, size = nnAX[8,1], prob = pAY1_x111)/nnAX[8,1];


	OBJ    <- matrix(999, gridgamma0, gridgamma);
	umeffs_x000_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x001_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x010_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x100_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x011_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x101_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x110_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x111_npmle <- matrix(-999, gridgamma0, gridgamma);

	lmeffs_x000_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x001_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x010_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x100_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x011_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x101_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x110_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x111_npmle <- matrix(999, gridgamma0, gridgamma);

	i <- 1;


###############################;
## RIDGE QUADRATIC PROGRAMMING;
###############################;

lambda	<- 1/(n*log(n));

MM	<- length(alpha0s);

for (j in 1:gridgamma) {;

	gamma <- gammas[j];


	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma, alpha0s);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma, alpha0s);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma, alpha0s);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma, alpha0s);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma, alpha0s);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma, alpha0s);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma, alpha0s);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma, alpha0s);

# 1st iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))));

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma, alpha0s, as.vector(prhos_x[, 8]));

# 2nd iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma, alpha0s, as.vector(prhos_x[, 8]));

# Final iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;


	};

	gamma0	  <- gammas[rank(OBJ) == 1];

	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma0, alpha0s);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma0, alpha0s);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma0, alpha0s);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma0, alpha0s);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma0, alpha0s);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma0, alpha0s);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma0, alpha0s);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma0, alpha0s);

# 1st iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))));

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));

# 2nd iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));

# Final iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	pAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	pAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	pAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	pAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	pAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	pAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	pAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	pAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));


# LINEAR PROGRAMMING;


	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma0, alphas);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma0, alphas);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma0, alphas);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma0, alphas);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma0, alphas);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma0, alphas);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma0, alphas);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma0, alphas);

	gamma	<- gamma0;
	i	<- 1;
	j	<- 1;

	## INDIVIDUAL MARGINAL EFFECTS
	# Using the same notation as for model parameters
		# x = (z_{1}, ..., z_{M})   M unknows
	# Constraints and objective function: 
		# Min/Max_{z} sum_{j}  sum_{m} z_{m} me(a_{m}|gamma,x^{j})
		# s.t  sum_{m} z_{m} pi(A_{i} | x^{j}, \gamma) = P(A_{i}| x^{j}, \gamma_{0})    nAY constraints
		#      sum_{m} z_{m} = 1                                                        1 constraints
		#      z_{m} >= 0								M constraints

		meffs_x000    <- ME(gamma, alphas, t(AX[1,]));
		meffs_x001    <- ME(gamma, alphas, t(AX[2,]));
		meffs_x010    <- ME(gamma, alphas, t(AX[3,]));
		meffs_x100    <- ME(gamma, alphas, t(AX[4,]));
		meffs_x011    <- ME(gamma, alphas, t(AX[5,]));
		meffs_x101    <- ME(gamma, alphas, t(AX[6,]));
		meffs_x110    <- ME(gamma, alphas, t(AX[7,]));
		meffs_x111    <- ME(gamma, alphas, t(AX[nrow(AX),]));


		## x = (0,0,0);

			fmeff_x000.obj  <- meffs_x000;
			fmeff_x000.dir  <- f_x.dir;
			fmeff_x000.rhs  <- t(cbind(t(pAY_x000),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x000.con	<- rbind(piAY_x000, t(rep(1, M)), diag(1, M));

			LPlmeff_x000     <- lp("min", fmeff_x000.obj, fmeff_x000.con, fmeff_x000.dir, fmeff_x000.rhs);
			lprhos_x000      <- matrix(LPlmeff_x000$solution, ncol = 1);

			if (LPlmeff_x000$status == 0) {; 
				lmeffs_x000_npmle[i,j]	<- LPlmeff_x000$objval;
				};
				

			LPumeff_x000   <- lp("max", fmeff_x000.obj, fmeff_x000.con, fmeff_x000.dir, fmeff_x000.rhs);
			uprhos_x000    <- matrix(LPumeff_x000$solution, ncol = 1);
			if (LPumeff_x000$status == 0) {; 
				umeffs_x000_npmle[i,j]	<- LPumeff_x000$objval;
				};
				


		## x = (1,1,1);

			fmeff_x111.obj   <- meffs_x111;
			fmeff_x111.dir   <- f_x.dir;
			fmeff_x111.rhs  	<- t(cbind(t(pAY_x111),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x111.con	<- rbind(piAY_x111, t(rep(1, M)), diag(1, M));

			LPlmeff_x111     <- lp("min", fmeff_x111.obj, fmeff_x111.con, fmeff_x111.dir, fmeff_x111.rhs);
			lprhos_x111      <- matrix(LPlmeff_x111$solution, ncol = 1);

			if (LPlmeff_x111$status == 0) {; 
				lmeffs_x111_npmle[i,j]	<- LPlmeff_x111$objval;
				};
				

			LPumeff_x111   <- lp("max", fmeff_x111.obj, fmeff_x111.con, fmeff_x111.dir, fmeff_x111.rhs);
			uprhos_x111    <- matrix(LPumeff_x111$solution, ncol = 1);
			if (LPumeff_x111$status == 0) {; 
				umeffs_x111_npmle[i,j]	<- LPumeff_x111$objval;
				};
				


	# Nonparametrically identified marginal effects;

		F1_x001			<- pAY_x001[4,1] + pAY_x001[6,1] + pAY_x001[7,1] + pAY_x001[8,1];
		F2_x001			<- pAY_x001[3,1] + pAY_x001[5,1] + pAY_x001[7,1] + pAY_x001[8,1];
		F3_x001			<- pAY_x001[2,1] + pAY_x001[5,1] + pAY_x001[6,1] + pAY_x001[8,1];
		umeffs_x001_npmle[i,j]	<- F3_x001 - F1_x001;
		lmeffs_x001_npmle[i,j]	<- F3_x001 - F2_x001;

		F1_x010			<- pAY_x010[4,1] + pAY_x010[6,1] + pAY_x010[7,1] + pAY_x010[8,1];
		F2_x010			<- pAY_x010[3,1] + pAY_x010[5,1] + pAY_x010[7,1] + pAY_x010[8,1];
		F3_x010			<- pAY_x010[2,1] + pAY_x010[5,1] + pAY_x010[6,1] + pAY_x010[8,1];
		umeffs_x010_npmle[i,j] 	<- F2_x010 - F1_x010;
		lmeffs_x010_npmle[i,j] 	<- F2_x010 - F3_x010;

		F1_x100			<- pAY_x100[4,1] + pAY_x100[6,1] + pAY_x100[7,1] + pAY_x100[8,1];
		F2_x100			<- pAY_x100[3,1] + pAY_x100[5,1] + pAY_x100[7,1] + pAY_x100[8,1];
		F3_x100			<- pAY_x100[2,1] + pAY_x100[5,1] + pAY_x100[6,1] + pAY_x100[8,1];
		umeffs_x100_npmle[i,j] 	<- F1_x100 - F2_x100;
		lmeffs_x100_npmle[i,j] 	<- F1_x100 - F3_x100;

		F1_x011			<- pAY_x011[4,1] + pAY_x011[6,1] + pAY_x011[7,1] + pAY_x011[8,1];
		F2_x011			<- pAY_x011[3,1] + pAY_x011[5,1] + pAY_x011[7,1] + pAY_x011[8,1];
		F3_x011			<- pAY_x011[2,1] + pAY_x011[5,1] + pAY_x011[6,1] + pAY_x011[8,1];
		umeffs_x011_npmle[i,j] 	<- F2_x011 - F1_x011;
		lmeffs_x011_npmle[i,j] 	<- F3_x011 - F1_x011;

		F1_x101			<- pAY_x101[4,1] + pAY_x101[6,1] + pAY_x101[7,1] + pAY_x101[8,1];
		F2_x101			<- pAY_x101[3,1] + pAY_x101[5,1] + pAY_x101[7,1] + pAY_x101[8,1];
		F3_x101			<- pAY_x101[2,1] + pAY_x101[5,1] + pAY_x101[6,1] + pAY_x101[8,1];
		umeffs_x101_npmle[i,j] 	<- F1_x101 - F2_x101;
		lmeffs_x101_npmle[i,j] 	<- F3_x101 - F2_x101;

		F1_x110			<- pAY_x110[4,1] + pAY_x110[6,1] + pAY_x110[7,1] + pAY_x110[8,1];
		F2_x110			<- pAY_x110[3,1] + pAY_x110[5,1] + pAY_x110[7,1] + pAY_x110[8,1];
		F3_x110			<- pAY_x110[2,1] + pAY_x110[5,1] + pAY_x110[6,1] + pAY_x110[8,1];
		umeffs_x110_npmle[i,j] 	<- F1_x110 - F3_x110;
		lmeffs_x110_npmle[i,j] 	<- F2_x110 - F3_x110;



	umeff_x000_npmle  <- apply(umeffs_x000_npmle,1,max);
	umeff_x001_npmle  <- apply(umeffs_x001_npmle,1,max);
	umeff_x010_npmle  <- apply(umeffs_x010_npmle,1,max);
	umeff_x100_npmle  <- apply(umeffs_x100_npmle,1,max);
	umeff_x011_npmle  <- apply(umeffs_x011_npmle,1,max);
	umeff_x101_npmle  <- apply(umeffs_x101_npmle,1,max);
	umeff_x110_npmle  <- apply(umeffs_x110_npmle,1,max);
	umeff_x111_npmle  <- apply(umeffs_x111_npmle,1,max);
	umeff_npmle	 <- pAX[1,1]*umeff_x000_npmle + pAX[2,1]*umeff_x001_npmle + pAX[3,1]*umeff_x010_npmle + pAX[4,1]*umeff_x100_npmle + pAX[5,1]*umeff_x011_npmle + pAX[6,1]*umeff_x101_npmle + pAX[7,1]*umeff_x110_npmle + pAX[8,1]*umeff_x111_npmle;

	lmeff_x000_npmle  <- apply(lmeffs_x000_npmle,1,min);
	lmeff_x001_npmle  <- apply(lmeffs_x001_npmle,1,min);
	lmeff_x010_npmle  <- apply(lmeffs_x010_npmle,1,min);
	lmeff_x100_npmle  <- apply(lmeffs_x100_npmle,1,min);
	lmeff_x011_npmle  <- apply(lmeffs_x011_npmle,1,min);
	lmeff_x101_npmle  <- apply(lmeffs_x101_npmle,1,min);
	lmeff_x110_npmle  <- apply(lmeffs_x110_npmle,1,min);
	lmeff_x111_npmle  <- apply(lmeffs_x111_npmle,1,min);
	lmeff_npmle	 <- pAX[1,1]*lmeff_x000_npmle + pAX[2,1]*lmeff_x001_npmle + pAX[3,1]*lmeff_x010_npmle + pAX[4,1]*lmeff_x100_npmle + pAX[5,1]*lmeff_x011_npmle + pAX[6,1]*lmeff_x101_npmle + pAX[7,1]*lmeff_x110_npmle + pAX[8,1]*lmeff_x111_npmle;


	umeff_x000.b  	<- c(umeff_x000.b, umeff_x000_npmle);
	umeff_x001.b  	<- c(umeff_x001.b, umeff_x001_npmle);
	umeff_x010.b  	<- c(umeff_x010.b, umeff_x010_npmle);
	umeff_x100.b  	<- c(umeff_x100.b, umeff_x100_npmle);
	umeff_x011.b  	<- c(umeff_x011.b, umeff_x011_npmle);
	umeff_x101.b  	<- c(umeff_x101.b, umeff_x101_npmle);
	umeff_x110.b  	<- c(umeff_x110.b, umeff_x110_npmle);
	umeff_x111.b  	<- c(umeff_x111.b, umeff_x111_npmle);
	umeff.b		<- c(umeff.b, umeff_npmle);

	lmeff_x000.b  	<- c(lmeff_x000.b, lmeff_x000_npmle);
	lmeff_x001.b  	<- c(lmeff_x001.b, lmeff_x001_npmle);
	lmeff_x010.b  	<- c(lmeff_x010.b, lmeff_x010_npmle);
	lmeff_x100.b  	<- c(lmeff_x100.b, lmeff_x100_npmle);
	lmeff_x011.b  	<- c(lmeff_x011.b, lmeff_x011_npmle);
	lmeff_x101.b  	<- c(lmeff_x101.b, lmeff_x101_npmle);
	lmeff_x110.b  	<- c(lmeff_x110.b, lmeff_x110_npmle);
	lmeff_x111.b  	<- c(lmeff_x111.b, lmeff_x111_npmle);
	lmeff.b		<- c(lmeff.b, lmeff_npmle);

	gamma.b		<- c(gamma.b, gamma0);
	lgamma.b	<- c(lgamma.b, gamma0);
	ugamma.b	<- c(ugamma.b, gamma0);



	};

gamma.m		<- c(gamma.m, mean(gamma.b));
lgamma.m	<- c(lgamma.m, quantile(lgamma.b, alpha/2 - beta/2, na.rm = TRUE));
ugamma.m	<- c(ugamma.m, quantile(ugamma.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x000.m	<- c(lmeff_x000.m, quantile(lmeff_x000.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x000.m	<- c(umeff_x000.m, quantile(umeff_x000.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x001.m	<- c(lmeff_x001.m, quantile(lmeff_x001.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x001.m	<- c(umeff_x001.m, quantile(umeff_x001.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x010.m	<- c(lmeff_x010.m, quantile(lmeff_x010.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x010.m	<- c(umeff_x010.m, quantile(umeff_x010.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x100.m	<- c(lmeff_x100.m, quantile(lmeff_x100.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x100.m	<- c(umeff_x100.m, quantile(umeff_x100.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x011.m	<- c(lmeff_x011.m, quantile(lmeff_x011.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x011.m	<- c(umeff_x011.m, quantile(umeff_x011.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x101.m	<- c(lmeff_x101.m, quantile(lmeff_x101.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x101.m	<- c(umeff_x101.m, quantile(umeff_x101.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x110.m	<- c(lmeff_x110.m, quantile(lmeff_x110.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x110.m	<- c(umeff_x110.m, quantile(umeff_x110.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff_x111.m	<- c(lmeff_x111.m, quantile(lmeff_x111.b, alpha/2 - beta/2, na.rm = TRUE));
umeff_x111.m	<- c(umeff_x111.m, quantile(umeff_x111.b, 1 - alpha/2 + beta/2, na.rm = TRUE));
lmeff.m		<- c(lmeff.m, quantile(lmeff.b, alpha/2 - beta/2, na.rm = TRUE));
umeff.m		<- c(umeff.m, quantile(umeff.b, 1 - alpha/2 + beta/2, na.rm = TRUE));


};



mean(gamma.m);
min(lgamma.m);
max(ugamma.m);
min(lmeff_x000.m);
max(umeff_x000.m);
min(lmeff_x001.m);
max(umeff_x001.m);
min(lmeff_x010.m);
max(umeff_x010.m);
min(lmeff_x100.m);
max(umeff_x100.m);
min(lmeff_x011.m);
max(umeff_x011.m);
min(lmeff_x101.m);
max(umeff_x101.m);
min(lmeff_x110.m);
max(umeff_x110.m);
min(lmeff_x111.m);
max(umeff_x111.m);
min(lmeff.m);
max(umeff.m);


# 95% Confidence interval;

# Parameter;
print(min(lgamma.m));
print(max(ugamma.m));

# AME;
print(min(lmeff.m));
print(max(umeff.m));


detach(data);


############################;
### MODIFIED PROJECTION;
############################;


library(stats);
library(lpSolve);
library(quadprog);
library(foreign);

# Reading dataset;
data	<- read.dta("data_static_T3.dta");
n	<- nrow(data);
attach(data);

# Parameters of the problem

TT	<- 3;
alpha0s <- c(-Inf,4*(-10:10)/10,Inf);
alphas 	<- c(-Inf,(-80:80)/10,Inf);
M0	<- length(alpha0s);
M      	<- length(alphas);

A      <- t(cbind(c(0,0,0), c(0,0,1), c(0,1,0), c(1,0,0), c(0,1,1), c(1,0,1), c(1,1,0), c(1,1,1)));
AY     <- A;
AX     <- A;

pAX	    <- matrix(0, nrow = 2^TT, ncol = 1);
pAX[1,1]    <- mean((x==1));
pAX[2,1]    <- mean((x==2));
pAX[3,1]    <- mean((x==3));
pAX[4,1]    <- mean((x==4));
pAX[5,1]    <- mean((x==5));
pAX[6,1]    <- mean((x==6));
pAX[7,1]    <- mean((x==7));
pAX[8,1]    <- mean((x==8));

nAX	<- 2^TT;
nAY	<- 2^TT;

gridgamma0 <- 1;

set.seed(888);   

nB	 	<- 5000;
alpha		<- .05;

gamma.b		<- NULL;
l.gamma.b	<- NULL;
u.gamma.b	<- NULL;

umeff_x000.b	<- NULL;
lmeff_x000.b	<- NULL;
umeff_x001.b	<- NULL;
lmeff_x001.b	<- NULL;
umeff_x010.b	<- NULL;
lmeff_x010.b	<- NULL;
umeff_x100.b	<- NULL;
lmeff_x100.b	<- NULL;
umeff_x011.b	<- NULL;
lmeff_x011.b	<- NULL;
umeff_x101.b	<- NULL;
lmeff_x101.b	<- NULL;
umeff_x110.b	<- NULL;
lmeff_x110.b	<- NULL;
umeff_x111.b	<- NULL;
lmeff_x111.b	<- NULL;

umeff.b		<- NULL;
lmeff.b		<- NULL;

gammas	  <- c(-200:10)/200;
gridgamma <- length(gammas);

f.obj <- t(cbind(t(rep(1,nAY*nAX+nAX)),t(rep(0,nAX*M))));
f.dir <- t(cbind(t(rep("=",nAY*nAX+nAX)),t(rep(">=", nAX*(nAY+1+M)))));
f_x.dir <- t(cbind(t(rep("=",nAY+1)),t(rep(">=", M))));

pAX0		<- pAX;

reject		<- 0;

for (b in 1:nB) {;

chisqtest	<- 0;
	
while (chisqtest == 0) {;
	
	pAX1		<- pAX0;	# Fixes probabilities of X to the observed probabilities in the dataset
	nnAX		<- n*pAX1;
	pAY_x000	<- rmultinom(1, size = nnAX[1,1], prob = PAY(x,y,1,TT))/nnAX[1,1];
	pAY_x001	<- rmultinom(1, size = nnAX[2,1], prob = PAY(x,y,2,TT))/nnAX[2,1];
	pAY_x010	<- rmultinom(1, size = nnAX[3,1], prob = PAY(x,y,3,TT))/nnAX[3,1];
	pAY_x100	<- rmultinom(1, size = nnAX[4,1], prob = PAY(x,y,4,TT))/nnAX[4,1];
	pAY_x011	<- rmultinom(1, size = nnAX[5,1], prob = PAY(x,y,5,TT))/nnAX[5,1];
	pAY_x101	<- rmultinom(1, size = nnAX[6,1], prob = PAY(x,y,6,TT))/nnAX[6,1];
	pAY_x110	<- rmultinom(1, size = nnAX[7,1], prob = PAY(x,y,7,TT))/nnAX[7,1];
	pAY_x111	<- rmultinom(1, size = nnAX[8,1], prob = PAY(x,y,8,TT))/nnAX[8,1];
	chisqstat	<- n * ( sum( (pAX1[1,1] * pAY_x000 - pAX0[1,1] * PAY(x,y,1,TT))^2 / (pAX0[1,1] * PAY(x,y,1,TT))) +  sum( (pAX1[2,1] * pAY_x001[-2,] - pAX0[2,1] * PAY(x,y,2,TT)[-2,])^2 / (pAX0[2,1] * PAY(x,y,2,TT)[-2,])) +  sum( (pAX1[3,1] * pAY_x010 - pAX0[3,1] * PAY(x,y,3,TT))^2 / (pAX0[3,1] * PAY(x,y,3,TT))) +  sum( (pAX1[4,1] * pAY_x100 - pAX0[4,1] * PAY(x,y,4,TT))^2 / (pAX0[4,1] * PAY(x,y,4,TT))) +  sum( (pAX1[5,1] * pAY_x011 - pAX0[5,1] * PAY(x,y,5,TT))^2 / (pAX0[5,1] * PAY(x,y,5,TT))) +  sum( (pAX1[6,1] * pAY_x101 - pAX0[6,1] * PAY(x,y,6,TT))^2 / (pAX0[6,1] * PAY(x,y,6,TT))) +  sum( (pAX1[7,1] * pAY_x110 - pAX0[7,1] * PAY(x,y,7,TT))^2 / (pAX0[7,1] * PAY(x,y,7,TT))) +  sum( (pAX1[8,1] * pAY_x111 - pAX0[8,1] * PAY(x,y,8,TT))^2 / (pAX0[8,1] * PAY(x,y,8,TT))) );
	if ( (chisqstat < qchisq(1-alpha, nAX*(nAY-1))) & (chisqstat > qchisq(1-2*alpha, nAX*(nAY-1))) ) chisqtest <- 1;
	if ( chisqstat >= qchisq(1-alpha, nAX*(nAY-1)) | chisqstat <= qchisq(1-2*alpha, nAX*(nAY-1)) ) reject <- reject + 1;
	};

	OBJ    <- matrix(999, gridgamma0, gridgamma);
	umeffs_x000_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x001_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x010_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x100_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x011_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x101_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x110_npmle <- matrix(-999, gridgamma0, gridgamma);
	umeffs_x111_npmle <- matrix(-999, gridgamma0, gridgamma);

	lmeffs_x000_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x001_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x010_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x100_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x011_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x101_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x110_npmle <- matrix(999, gridgamma0, gridgamma);
	lmeffs_x111_npmle <- matrix(999, gridgamma0, gridgamma);

	i <- 1;


###############################;
## RIDGE QUADRATIC PROGRAMMING;
###############################;

lambda	<- 1/(n*log(n));
MM	<- length(alpha0s);

for (j in 1:gridgamma) {;

	gamma <- gammas[j];


	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma, alpha0s);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma, alpha0s);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma, alpha0s);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma, alpha0s);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma, alpha0s);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma, alpha0s);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma, alpha0s);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma, alpha0s);

# 1st iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))));

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma, alpha0s, as.vector(prhos_x[, 8]));

# 2nd iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;
                        
	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma, alpha0s, as.vector(prhos_x[, 8]));

# Final iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);
	OBJ[i,j]  <- 2*n*QP$value;


	};

	gamma0	  <- gammas[rank(OBJ) == 1];

	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma0, alpha0s);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma0, alpha0s);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma0, alpha0s);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma0, alpha0s);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma0, alpha0s);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma0, alpha0s);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma0, alpha0s);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma0, alpha0s);

# 1st iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))));

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));

# 2nd iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);


	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	ppAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	ppAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	ppAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	ppAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	ppAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	ppAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	ppAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	ppAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));

# Final iteration;

	aa1_x000    <- sqrt(pAX[1, ]) * cbind((piAY_x000 - kronecker(pAY_x000, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x000), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 7*MM));
	aa1_x001    <- sqrt(pAX[2, ]) * cbind(matrix(0, nAY, MM),   (piAY_x001 - kronecker(pAY_x001, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x001), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 6*MM));
	aa1_x010    <- sqrt(pAX[3, ]) * cbind(matrix(0, nAY, 2*MM), (piAY_x010 - kronecker(pAY_x010, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x010), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 5*MM));
	aa1_x100    <- sqrt(pAX[4, ]) * cbind(matrix(0, nAY, 3*MM), (piAY_x100 - kronecker(pAY_x100, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x100), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 4*MM));
	aa1_x011    <- sqrt(pAX[5, ]) * cbind(matrix(0, nAY, 4*MM), (piAY_x011 - kronecker(pAY_x011, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x011), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 3*MM));
	aa1_x101    <- sqrt(pAX[6, ]) * cbind(matrix(0, nAY, 5*MM), (piAY_x101 - kronecker(pAY_x101, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x101), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, 2*MM));
	aa1_x110    <- sqrt(pAX[7, ]) * cbind(matrix(0, nAY, 6*MM), (piAY_x110 - kronecker(pAY_x110, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x110), matrix(1, nrow = 1, ncol = MM)), matrix(0, nAY, MM));
	aa1_x111    <- sqrt(pAX[8, ]) * cbind(matrix(0, nAY, 7*MM), (piAY_x111 - kronecker(pAY_x111, matrix(1, nrow = 1, ncol = MM))) / kronecker(sqrt(ppAY_x111), matrix(1, nrow = 1, ncol = MM)) );

	A   <- rbind(aa1_x000, aa1_x001, aa1_x010, aa1_x100, aa1_x011, aa1_x101, aa1_x110, aa1_x111);
	B   <- rbind(0 * (pAY_x000 - piAY_x000[ ,1]), 0 * (pAY_x001 - piAY_x001[ ,1]), 0 * (pAY_x010 - piAY_x010[ ,1]), 0 * (pAY_x100 - piAY_x100[ ,1]), 
		     0 * (pAY_x011 - piAY_x011[ ,1]), 0 * (pAY_x101 - piAY_x101[ ,1]), 0 * (pAY_x110 - piAY_x110[ ,1]), 0 * (pAY_x111 - piAY_x111[ ,1]) );
 
	Dmat   <- t(A) %*%  A + lambda * diag(1, ncol = ncol(A), nrow = ncol(A));
	dvec   <- t(A) %*% B;
	Amat   <- t(rbind(kronecker(diag(-1, nrow = nAX), matrix(1, nrow=1, ncol = MM)), diag(1, nrow = nAX*MM)));
	bvec   <- rbind(matrix(-1, nrow = nAX, ncol = 1), matrix(0, nrow = nAX*MM, ncol = 1));
	meq    <- nAX;
 
	QP <- solve.QP(Dmat, dvec, Amat, bvec, meq);

	prhos_x   <- matrix(QP$solution, nrow = MM, ncol = nAX, byrow = FALSE);

	pAY_x000     <- PAYm(AY, t(AX[1,]), gamma0, alpha0s, as.vector(prhos_x[, 1]));
	pAY_x001     <- PAYm(AY, t(AX[2,]), gamma0, alpha0s, as.vector(prhos_x[, 2]));
	pAY_x010     <- PAYm(AY, t(AX[3,]), gamma0, alpha0s, as.vector(prhos_x[, 3]));
	pAY_x100     <- PAYm(AY, t(AX[4,]), gamma0, alpha0s, as.vector(prhos_x[, 4]));
	pAY_x011     <- PAYm(AY, t(AX[5,]), gamma0, alpha0s, as.vector(prhos_x[, 5]));
	pAY_x101     <- PAYm(AY, t(AX[6,]), gamma0, alpha0s, as.vector(prhos_x[, 6]));
	pAY_x110     <- PAYm(AY, t(AX[7,]), gamma0, alpha0s, as.vector(prhos_x[, 7]));
	pAY_x111     <- PAYm(AY, t(AX[8,]), gamma0, alpha0s, as.vector(prhos_x[, 8]));

# LINEAR PROGRAMMING;


	piAY_x000  <- PIAY(AY, t(AX[1,]), gamma0, alphas);
	piAY_x001  <- PIAY(AY, t(AX[2,]), gamma0, alphas);
	piAY_x010  <- PIAY(AY, t(AX[3,]), gamma0, alphas);
	piAY_x100  <- PIAY(AY, t(AX[4,]), gamma0, alphas);
	piAY_x011  <- PIAY(AY, t(AX[5,]), gamma0, alphas);
	piAY_x101  <- PIAY(AY, t(AX[6,]), gamma0, alphas);
	piAY_x110  <- PIAY(AY, t(AX[7,]), gamma0, alphas);
	piAY_x111  <- PIAY(AY, t(AX[8,]), gamma0, alphas);

	gamma	<- gamma0;
	i	<- 1;
	j	<- 1;

	## INDIVIDUAL MARGINAL EFFECTS
	# Using the same notation as for model parameters
		# x = (z_{1}, ..., z_{M})   M unknows
	# Constraints and objective function: 
		# Min/Max_{z} sum_{j}  sum_{m} z_{m} me(a_{m}|gamma,x^{j})
		# s.t  sum_{m} z_{m} pi(A_{i} | x^{j}, \gamma) = P(A_{i}| x^{j}, \gamma_{0})    nAY constraints
		#      sum_{m} z_{m} = 1                                                        1 constraints
		#      z_{m} >= 0								M constraints

		meffs_x000    <- ME(gamma, alphas, t(AX[1,]));
		meffs_x001    <- ME(gamma, alphas, t(AX[2,]));
		meffs_x010    <- ME(gamma, alphas, t(AX[3,]));
		meffs_x100    <- ME(gamma, alphas, t(AX[4,]));
		meffs_x011    <- ME(gamma, alphas, t(AX[5,]));
		meffs_x101    <- ME(gamma, alphas, t(AX[6,]));
		meffs_x110    <- ME(gamma, alphas, t(AX[7,]));
		meffs_x111    <- ME(gamma, alphas, t(AX[nrow(AX),]));


		## x = (0,0,0);

			fmeff_x000.obj  <- meffs_x000;
			fmeff_x000.dir  <- f_x.dir;
			fmeff_x000.rhs  <- t(cbind(t(pAY_x000),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x000.con	<- rbind(piAY_x000, t(rep(1, M)), diag(1, M));

			LPlmeff_x000     <- lp("min", fmeff_x000.obj, fmeff_x000.con, fmeff_x000.dir, fmeff_x000.rhs);
			lprhos_x000      <- matrix(LPlmeff_x000$solution, ncol = 1);

			if (LPlmeff_x000$status == 0) {; 
				lmeffs_x000_npmle[i,j]	<- LPlmeff_x000$objval;
				};
				
			if (LPlmeff_x000$status == 2) {;
				lmeffs_x000_npmle[i,j]	<- NaN;
				};

			LPumeff_x000   <- lp("max", fmeff_x000.obj, fmeff_x000.con, fmeff_x000.dir, fmeff_x000.rhs);
			uprhos_x000    <- matrix(LPumeff_x000$solution, ncol = 1);
			if (LPumeff_x000$status == 0) {; 
				umeffs_x000_npmle[i,j]	<- LPumeff_x000$objval;
				};
				
			if (LPumeff_x000$status == 2) {;
				umeffs_x000_npmle[i,j]	<- NaN;
				};


		## x = (1,1,1);

			fmeff_x111.obj   <- meffs_x111;
			fmeff_x111.dir   <- f_x.dir;
			fmeff_x111.rhs  	<- t(cbind(t(pAY_x111),t(rep(1, 1)), t(rep(0, M))));
	                fmeff_x111.con	<- rbind(piAY_x111, t(rep(1, M)), diag(1, M));

			LPlmeff_x111     <- lp("min", fmeff_x111.obj, fmeff_x111.con, fmeff_x111.dir, fmeff_x111.rhs);
			lprhos_x111      <- matrix(LPlmeff_x111$solution, ncol = 1);

			if (LPlmeff_x111$status == 0) {; 
				lmeffs_x111_npmle[i,j]	<- LPlmeff_x111$objval;
				};
				
			if (LPlmeff_x111$status == 2) {;
				lmeffs_x111_npmle[i,j]	<- NaN;
				};

			LPumeff_x111   <- lp("max", fmeff_x111.obj, fmeff_x111.con, fmeff_x111.dir, fmeff_x111.rhs);
			uprhos_x111    <- matrix(LPumeff_x111$solution, ncol = 1);
			if (LPumeff_x111$status == 0) {; 
				umeffs_x111_npmle[i,j]	<- LPumeff_x111$objval;
				};
				
			if (LPumeff_x111$status == 2) {;
				umeffs_x111_npmle[i,j]	<- NaN;
				};


	# Nonparametrically identified marginal effects;

		F1_x001			<- pAY_x001[4,1] + pAY_x001[6,1] + pAY_x001[7,1] + pAY_x001[8,1];
		F2_x001			<- pAY_x001[3,1] + pAY_x001[5,1] + pAY_x001[7,1] + pAY_x001[8,1];
		F3_x001			<- pAY_x001[2,1] + pAY_x001[5,1] + pAY_x001[6,1] + pAY_x001[8,1];
		umeffs_x001_npmle[i,j]	<- F3_x001 - F1_x001;
		lmeffs_x001_npmle[i,j]	<- F3_x001 - F2_x001;

		F1_x010			<- pAY_x010[4,1] + pAY_x010[6,1] + pAY_x010[7,1] + pAY_x010[8,1];
		F2_x010			<- pAY_x010[3,1] + pAY_x010[5,1] + pAY_x010[7,1] + pAY_x010[8,1];
		F3_x010			<- pAY_x010[2,1] + pAY_x010[5,1] + pAY_x010[6,1] + pAY_x010[8,1];
		umeffs_x010_npmle[i,j] 	<- F2_x010 - F1_x010;
		lmeffs_x010_npmle[i,j] 	<- F2_x010 - F3_x010;

		F1_x100			<- pAY_x100[4,1] + pAY_x100[6,1] + pAY_x100[7,1] + pAY_x100[8,1];
		F2_x100			<- pAY_x100[3,1] + pAY_x100[5,1] + pAY_x100[7,1] + pAY_x100[8,1];
		F3_x100			<- pAY_x100[2,1] + pAY_x100[5,1] + pAY_x100[6,1] + pAY_x100[8,1];
		umeffs_x100_npmle[i,j] 	<- F1_x100 - F2_x100;
		lmeffs_x100_npmle[i,j] 	<- F1_x100 - F3_x100;

		F1_x011			<- pAY_x011[4,1] + pAY_x011[6,1] + pAY_x011[7,1] + pAY_x011[8,1];
		F2_x011			<- pAY_x011[3,1] + pAY_x011[5,1] + pAY_x011[7,1] + pAY_x011[8,1];
		F3_x011			<- pAY_x011[2,1] + pAY_x011[5,1] + pAY_x011[6,1] + pAY_x011[8,1];
		umeffs_x011_npmle[i,j] 	<- F2_x011 - F1_x011;
		lmeffs_x011_npmle[i,j] 	<- F3_x011 - F1_x011;

		F1_x101			<- pAY_x101[4,1] + pAY_x101[6,1] + pAY_x101[7,1] + pAY_x101[8,1];
		F2_x101			<- pAY_x101[3,1] + pAY_x101[5,1] + pAY_x101[7,1] + pAY_x101[8,1];
		F3_x101			<- pAY_x101[2,1] + pAY_x101[5,1] + pAY_x101[6,1] + pAY_x101[8,1];
		umeffs_x101_npmle[i,j] 	<- F1_x101 - F2_x101;
		lmeffs_x101_npmle[i,j] 	<- F3_x101 - F2_x101;

		F1_x110			<- pAY_x110[4,1] + pAY_x110[6,1] + pAY_x110[7,1] + pAY_x110[8,1];
		F2_x110			<- pAY_x110[3,1] + pAY_x110[5,1] + pAY_x110[7,1] + pAY_x110[8,1];
		F3_x110			<- pAY_x110[2,1] + pAY_x110[5,1] + pAY_x110[6,1] + pAY_x110[8,1];
		umeffs_x110_npmle[i,j] 	<- F1_x110 - F3_x110;
		lmeffs_x110_npmle[i,j] 	<- F2_x110 - F3_x110;



	umeff_x000_npmle  <- apply(umeffs_x000_npmle,1,max);
	umeff_x001_npmle  <- apply(umeffs_x001_npmle,1,max);
	umeff_x010_npmle  <- apply(umeffs_x010_npmle,1,max);
	umeff_x100_npmle  <- apply(umeffs_x100_npmle,1,max);
	umeff_x011_npmle  <- apply(umeffs_x011_npmle,1,max);
	umeff_x101_npmle  <- apply(umeffs_x101_npmle,1,max);
	umeff_x110_npmle  <- apply(umeffs_x110_npmle,1,max);
	umeff_x111_npmle  <- apply(umeffs_x111_npmle,1,max);
	umeff_npmle	 <- pAX[1,1]*umeff_x000_npmle + pAX[2,1]*umeff_x001_npmle + pAX[3,1]*umeff_x010_npmle + pAX[4,1]*umeff_x100_npmle + pAX[5,1]*umeff_x011_npmle + pAX[6,1]*umeff_x101_npmle + pAX[7,1]*umeff_x110_npmle + pAX[8,1]*umeff_x111_npmle;

	lmeff_x000_npmle  <- apply(lmeffs_x000_npmle,1,min);
	lmeff_x001_npmle  <- apply(lmeffs_x001_npmle,1,min);
	lmeff_x010_npmle  <- apply(lmeffs_x010_npmle,1,min);
	lmeff_x100_npmle  <- apply(lmeffs_x100_npmle,1,min);
	lmeff_x011_npmle  <- apply(lmeffs_x011_npmle,1,min);
	lmeff_x101_npmle  <- apply(lmeffs_x101_npmle,1,min);
	lmeff_x110_npmle  <- apply(lmeffs_x110_npmle,1,min);
	lmeff_x111_npmle  <- apply(lmeffs_x111_npmle,1,min);
	lmeff_npmle	 <- pAX[1,1]*lmeff_x000_npmle + pAX[2,1]*lmeff_x001_npmle + pAX[3,1]*lmeff_x010_npmle + pAX[4,1]*lmeff_x100_npmle + pAX[5,1]*lmeff_x011_npmle + pAX[6,1]*lmeff_x101_npmle + pAX[7,1]*lmeff_x110_npmle + pAX[8,1]*lmeff_x111_npmle;


	umeff_x000.b  	<- c(umeff_x000.b, umeff_x000_npmle);
	umeff_x001.b  	<- c(umeff_x001.b, umeff_x001_npmle);
	umeff_x010.b  	<- c(umeff_x010.b, umeff_x010_npmle);
	umeff_x100.b  	<- c(umeff_x100.b, umeff_x100_npmle);
	umeff_x011.b  	<- c(umeff_x011.b, umeff_x011_npmle);
	umeff_x101.b  	<- c(umeff_x101.b, umeff_x101_npmle);
	umeff_x110.b  	<- c(umeff_x110.b, umeff_x110_npmle);
	umeff_x111.b  	<- c(umeff_x111.b, umeff_x111_npmle);
	umeff.b		<- c(umeff.b, umeff_npmle);

	lmeff_x000.b  	<- c(lmeff_x000.b, lmeff_x000_npmle);
	lmeff_x001.b  	<- c(lmeff_x001.b, lmeff_x001_npmle);
	lmeff_x010.b  	<- c(lmeff_x010.b, lmeff_x010_npmle);
	lmeff_x100.b  	<- c(lmeff_x100.b, lmeff_x100_npmle);
	lmeff_x011.b  	<- c(lmeff_x011.b, lmeff_x011_npmle);
	lmeff_x101.b  	<- c(lmeff_x101.b, lmeff_x101_npmle);
	lmeff_x110.b  	<- c(lmeff_x110.b, lmeff_x110_npmle);
	lmeff_x111.b  	<- c(lmeff_x111.b, lmeff_x111_npmle);
	lmeff.b		<- c(lmeff.b, lmeff_npmle);

	gamma.b		<- c(gamma.b, gamma0);

	};


min(gamma.b);
max(gamma.b);
min(lmeff_x000.b);
max(umeff_x000.b);
min(lmeff_x001.b);
max(umeff_x001.b);
min(lmeff_x010.b);
max(umeff_x010.b);
min(lmeff_x100.b);
max(umeff_x100.b);
min(lmeff_x011.b);
max(umeff_x011.b);
min(lmeff_x101.b);
max(umeff_x101.b);
min(lmeff_x110.b);
max(umeff_x110.b);
min(lmeff_x111.b, na.rm = TRUE);
max(umeff_x111.b, na.rm = TRUE);
min(lmeff.b, na.rm = TRUE);
max(umeff.b, na.rm = TRUE);

# 95% Confidence interval;

# Parameter;
print(min(gamma.b));
print(max(gamma.b));

# AME;
print(min(lmeff.b, na.rm = TRUE));
print(max(umeff.b, na.rm = TRUE));


detach(data);


